from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from genericm2m.models import RelatedObject, RelatedObjectsDescriptor, GFKOptimizedQuerySet
from genericm2m.genericm2m_tests.models import (
    Food, Beverage, Person, RelatedBeverage, Boring, AnotherRelatedObject, Note
)


class RelationsTestCase(TestCase):
    def setUp(self):
        self.pizza = Food.objects.create(name='pizza')
        self.sandwich = Food.objects.create(name='sandwich')
        self.cereal = Food.objects.create(name='cereal')

        self.soda = Beverage.objects.create(name='soda')
        self.beer = Beverage.objects.create(name='beer')
        self.milk = Beverage.objects.create(name='milk')

        self.mario = Person.objects.create(name='mario')
        self.sam = Person.objects.create(name='sam')
        self.chocula = Person.objects.create(name='chocula')

        self.table = Boring.objects.create(name='table')
        self.chair = Boring.objects.create(name='chair')

    def assertRelatedEqual(self, rel_qs, tups, from_field='parent',
                           to_field='object'):
        rel_tup = [
            (getattr(rel_obj, from_field), getattr(rel_obj, to_field)) \
            for rel_obj in rel_qs
        ]
        self.assertEqual(rel_tup, list(tups))

    def test_connect(self):
        """
        Connect model instances to various other model instances, then query
        the manager and check the queryset returned is correct
        """
        self.pizza.related.connect(self.soda)
        self.pizza.related.connect(self.beer)
        self.pizza.related.connect(self.mario)

        self.soda.related.connect(self.pizza)
        self.soda.related.connect(self.beer)

        related = self.pizza.related.all()
        self.assertRelatedEqual(related, (
            (self.pizza, self.mario),
            (self.pizza, self.beer),
            (self.pizza, self.soda),
        ))

        self.sandwich.related.connect(self.soda)
        self.sandwich.related.connect(self.milk)

        related = self.sandwich.related.all()
        self.assertRelatedEqual(related, (
            (self.sandwich, self.milk),
            (self.sandwich, self.soda),
        ))

        related = self.cereal.related.all()
        self.assertRelatedEqual(related, ())

        related = self.soda.related.all()
        self.assertRelatedEqual(related, (
            (self.soda, self.beer),
            (self.soda, self.pizza),
        ))

        self.sandwich.related.connect(self.table)

        related = self.sandwich.related.all()
        self.assertRelatedEqual(related, (
            (self.sandwich, self.table),
            (self.sandwich, self.milk),
            (self.sandwich, self.soda),
        ))

    def test_related_to(self):
        """
        Check the back-side of the double-GFK, note: this only works on objects
        that have a RelatedObjectsDescriptor() pointing to the same model
        class, in this case the default `RelatedObject`
        """
        self.pizza.related.connect(self.soda)
        self.pizza.related.connect(self.beer)
        self.pizza.related.connect(self.table)
        self.sandwich.related.connect(self.soda)
        self.sandwich.related.connect(self.milk)
        self.mario.related.connect(self.soda)
        self.soda.related.connect(self.pizza)

        related = self.soda.related.related_to()
        self.assertRelatedEqual(related, (
            (self.mario, self.soda),
            (self.sandwich, self.soda),
            (self.pizza, self.soda),
        ))

        related = self.beer.related.related_to()
        self.assertRelatedEqual(related, (
            (self.pizza, self.beer),
        ))

        related = self.milk.related.related_to()
        self.assertRelatedEqual(related, (
            (self.sandwich, self.milk),
        ))

        related = self.pizza.related.related_to()
        self.assertRelatedEqual(related, (
            (self.soda, self.pizza),
        ))

    def test_symmetrical(self):
        self.pizza.related.connect(self.soda)
        self.pizza.related.connect(self.beer)
        self.pizza.related.connect(self.table)
        self.sandwich.related.connect(self.soda)
        self.sandwich.related.connect(self.milk)
        self.mario.related.connect(self.soda)
        self.soda.related.connect(self.pizza)

        related = self.soda.related.symmetrical().order_by('id')
        self.assertRelatedEqual(related, (
            (self.pizza, self.soda),
            (self.sandwich, self.soda),
            (self.mario, self.soda),
            (self.soda, self.pizza),
        ))

        related = self.beer.related.symmetrical()
        self.assertRelatedEqual(related, (
            (self.pizza, self.beer),
        ))

    def test_manager_methods(self):
        """
        Since the RelatedObjectsDescriptor behaves like a dynamic manager (much
        the same as Django's ForeignRelatedObjectsDescriptor) test to ensure
        that the manager behaves as expected and correctly implements all the
        basic FK methods
        """
        # connect pizza to soda and grab the newly-created RelatedObject
        self.pizza.related.connect(self.soda)
        rel_obj = RelatedObject.objects.all()[0]

        # connect cereal to milk (this is just to make sure that anything
        # modified on one Food object doesn't affect another Food object
        self.cereal.related.connect(self.milk)

        # create a new RelatedObject but do not save it yet -- note that it does
        # not have `parent_object` set
        new_rel_obj = RelatedObject(object=self.beer)

        # add this related object to pizza, parent_object gets set and it will
        # show up in the queryset as expected
        self.pizza.related.add(new_rel_obj)
        self.assertRelatedEqual(self.pizza.related.all(), (
            (self.pizza, self.beer),
            (self.pizza, self.soda),
        ))

        # remove the original RelatedObject `rel_obj`, which was the connection
        # from pizza -> soda
        self.pizza.related.remove(rel_obj)
        self.assertRelatedEqual(self.pizza.related.all(), (
            (self.pizza, self.beer),
        ))

        # make sure clearing pizza's related queryset works
        self.pizza.related.clear()
        self.assertRelatedEqual(self.pizza.related.all(), ())

        # make sure clearing the pizza objects didn't affect cereal
        self.assertRelatedEqual(self.cereal.related.all(), (
            (self.cereal, self.milk),
        ))

        # there should be just one row in the table
        self.assertEqual(RelatedObject.objects.count(), 1)

    def test_model_level(self):
        """
        The RelatedObjectsDescriptor can work at the class-level as well and
        applies to all instances of the model - check that when connections are
        made between individual instances and then are queried via the class,
        that all connections are returned from that model type
        """
        self.pizza.related.connect(self.beer)
        self.cereal.related.connect(self.milk)

        self.mario.related.connect(self.pizza)
        self.sam.related.connect(self.beer)
        self.soda.related.connect(self.pizza)

        self.assertRelatedEqual(Food.related.all(), (
            (self.cereal, self.milk),
            (self.pizza, self.beer),
        ))

        self.assertRelatedEqual(Beverage.related.all(), (
            (self.soda, self.pizza),
        ))

        self.assertRelatedEqual(Person.related.all(), (
            (self.sam, self.beer),
            (self.mario, self.pizza),
        ))

    def test_custom_connect(self):
        """
        Mimic the test_connect() method, but instead use the custom descriptor,
        `related_beverages` which goes through the RelatedBeverage model
        """
        self.pizza.related_beverages.connect(self.soda)
        self.pizza.related_beverages.connect(self.beer)

        related = self.pizza.related_beverages.all()
        self.assertRelatedEqual(related, (
            (self.pizza, self.beer),
            (self.pizza, self.soda),
        ), 'food', 'beverage')

        self.sandwich.related_beverages.connect(self.soda)
        self.sandwich.related_beverages.connect(self.milk)

        related = self.sandwich.related_beverages.all()
        self.assertRelatedEqual(related, (
            (self.sandwich, self.milk),
            (self.sandwich, self.soda),
        ), 'food', 'beverage')

        related = self.cereal.related_beverages.all()
        self.assertRelatedEqual(related, ())

    def test_custom_model_manager(self):
        """
        Mimic the test_model_manager() method, but instead use the custom
        descriptor and through model
        """
        self.pizza.related_beverages.connect(self.soda)
        rel_obj = RelatedBeverage.objects.all()[0] # grab the new related obj

        self.cereal.related_beverages.connect(self.milk)

        new_rel_obj = RelatedBeverage(beverage=self.beer)

        self.pizza.related_beverages.add(new_rel_obj)
        self.assertRelatedEqual(self.pizza.related_beverages.all(), (
            (self.pizza, self.beer),
            (self.pizza, self.soda),
        ), 'food', 'beverage')

        self.pizza.related_beverages.remove(rel_obj)
        self.assertRelatedEqual(self.pizza.related_beverages.all(), (
            (self.pizza, self.beer),
        ), 'food', 'beverage')

        self.pizza.related_beverages.clear()
        self.assertRelatedEqual(self.pizza.related_beverages.all(), ())

        # make sure clearing the pizza objects didn't affect cereal
        self.assertRelatedEqual(self.cereal.related_beverages.all(), (
            (self.cereal, self.milk),
        ), 'food', 'beverage')

        self.assertEqual(RelatedBeverage.objects.count(), 1)

    def test_custom_model_level(self):
        """
        And lastly, test that the custom descriptor/through-model work as
        expected at the model-level (previous tests were instance-level)
        """
        self.pizza.related_beverages.connect(self.soda)
        self.pizza.related_beverages.connect(self.beer)
        self.sandwich.related_beverages.connect(self.soda)
        self.cereal.related_beverages.connect(self.milk)

        self.assertRelatedEqual(Food.related_beverages.all(), (
            (self.cereal, self.milk),
            (self.sandwich, self.soda),
            (self.pizza, self.beer),
            (self.pizza, self.soda),
        ), 'food', 'beverage')

    def test_generic_traversal(self):
        """
        Ensure that the RelatedObjectsDescriptor returns a GFKOptimizedQuerySet
        when the through model contains a GFK -- also check that the queryset's
        optimized lookup works as expected
        """
        self.pizza.related.connect(self.beer)
        self.pizza.related.connect(self.soda)
        self.pizza.related.connect(self.mario)

        # the manager returns instances of GFKOptimizedQuerySet
        related = self.pizza.related.all()
        self.assertEqual(type(related), GFKOptimizedQuerySet)

        # check the queryset is using the right field
        self.assertEqual(related.get_gfk().name, 'object')

        # the custom queryset's optimized lookup works correctly
        objects = related.generic_objects()
        self.assertEqual(objects, [self.mario, self.soda, self.beer])

        # check the reverse does not hold, documenting existing behavior since
        # it looks at only the "default" manager on the back-side
        related = self.soda.related.related_to()
        self.assertEqual(type(related), GFKOptimizedQuerySet)

        # check the queryset is using the right field
        self.assertEqual(related.get_gfk().name, 'parent')

        # the custom queryset's optimized lookup works correctly
        objects = related.generic_objects()
        self.assertEqual(objects, [self.pizza])

    def test_filtering(self):
        """
        Check that filtering on RelatedObject fields (or through model fields)
        works as expected
        """
        self.pizza.related.connect(self.beer, alias='bud lite')
        self.pizza.related.connect(self.soda, alias='pepsi')
        self.pizza.related.connect(self.mario)

        rel_qs = self.pizza.related.filter(alias='bud lite')
        self.assertRelatedEqual(rel_qs, (
            (self.pizza, self.beer),
        ))

        rel_qs = self.pizza.related.filter(object_type=ContentType.objects.get_for_model(Beverage))
        self.assertRelatedEqual(rel_qs, (
            (self.pizza, self.soda),
            (self.pizza, self.beer),
        ))

        rel_qs = self.beer.related.related_to().filter(alias='bud lite')
        self.assertRelatedEqual(rel_qs, (
            (self.pizza, self.beer),
        ))

    def test_custom_model_using_gfks(self):
        """
        Check that using a custom through model with GFKs works as expected
        (looking at models.py, Note uses `AnotherRelatedObject` as its through)
        """
        self.note_a = Note.objects.create(content='a')
        self.note_b = Note.objects.create(content='b')
        self.note_c = Note.objects.create(content='c')

        self.note_a.related.connect(self.pizza)
        self.note_a.related.connect(self.note_b)

        self.pizza.related.connect(self.note_b)

        # create some notes with custom attributes
        self.note_b.related.connect(self.cereal, alias='cereal note', description='lucky charms!')
        self.note_b.related.connect(self.milk, alias='milk note', description='goes good with cereal')

        # ensure that the queryset is using the correct model and automatically
        # determines that a GFKOptimizedQuerySet can be used
        queryset = self.note_a.related.all()
        self.assertEqual(queryset.model, AnotherRelatedObject)
        self.assertTrue(isinstance(queryset, GFKOptimizedQuerySet))

        related_a = self.note_a.related.all()
        self.assertRelatedEqual(related_a, (
            (self.note_a, self.pizza),
            (self.note_a, self.note_b),
        ))

        related_b = self.note_b.related.all()
        self.assertRelatedEqual(related_b, (
            (self.note_b, self.cereal),
            (self.note_b, self.milk),
        ))

        related_to = self.note_b.related.related_to()
        # note that pizza does not show up here even though it is related to note b
        # this is because that relationship was stored in a different table (RelatedObject)
        # as opposed to AnotherRelatedObject
        self.assertEqual(related_to.generic_objects(), [self.note_a])

        cereal_rel, milk_rel = related_b

        # check that the custom attributes were saved correctly
        self.assertEqual(cereal_rel.alias, 'cereal note')
        self.assertEqual(cereal_rel.description, 'lucky charms!')

        self.assertEqual(milk_rel.alias, 'milk note')
        self.assertEqual(milk_rel.description, 'goes good with cereal')

        # check that we can filter on fields as expected
        self.assertRelatedEqual(self.note_b.related.filter(alias='cereal note'), (
            (self.note_b, self.cereal),
        ))

        related_c = self.note_c.related.all()
        self.assertRelatedEqual(related_c, ())

        # lastly, check that the GFKOptimizedQuerySet returns the expected
        # results when doing the optimized lookup
        self.assertEqual(related_a.generic_objects(), [
            self.pizza, self.note_b
        ])

        self.assertEqual(related_b.generic_objects(), [
            self.cereal, self.milk
        ])

        self.assertEqual(related_c.generic_objects(), [])

    def test_generic_objects_filtered(self):
        """
        Get generic objects filtered by Model.
        """
        self.pizza.related.connect(self.beer)
        self.pizza.related.connect(self.soda)
        self.pizza.related.connect(self.mario)

        # Get all generic related content
        related = self.pizza.related.all()
        objects = related.generic_objects()
        self.assertEqual(objects, [self.mario, self.soda, self.beer])

        # Get Person generic related content only.
        related = self.pizza.related.all()
        objects = related.generic_objects(Person)
        self.assertEqual(objects, [self.mario])



#!/usr/bin/env python
import sys
from os.path import dirname, abspath

import django
from django.conf import settings


if len(sys.argv) > 1 and 'postgres' in sys.argv:
    sys.argv.remove('postgres')
    db_engine = 'django.db.backends.postgresql_psycopg2'
    db_name = 'test_main'
else:
    db_engine = 'django.db.backends.sqlite3'
    db_name = ''

if not settings.configured:
    settings.configure(
        DATABASES=dict(default=dict(ENGINE=db_engine, NAME=db_name)),
        INSTALLED_APPS = [
            'django.contrib.contenttypes',
            'genericm2m',
            'genericm2m.genericm2m_tests',
        ],
        MIDDLEWARE_CLASSES = (),
    )

from django.test.utils import get_runner

try:
    django.setup()
except AttributeError:
    pass

def runtests(*test_args):
    if not test_args:
        if sys.version_info[0] > 2:
            test_args = ['genericm2m.genericm2m_tests']
        else:
            test_args = ["genericm2m_tests"]
    parent = dirname(abspath(__file__))
    sys.path.insert(0, parent)
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=1, interactive=True)
    failures = test_runner.run_tests(test_args)
    sys.exit(failures)

if __name__ == '__main__':
    runtests(*sys.argv[1:])



from django.contrib import admin
from nested_admin import NestedStackedInline, NestedModelAdmin

from .models import (
    TestAdminWidgetsRoot, TestAdminWidgetsM2M, TestAdminWidgetsRelated1,
    TestAdminWidgetsRelated2, TestAdminWidgetsA, TestAdminWidgetsB,
    TestAdminWidgetsC0, TestAdminWidgetsC1)


class TestAdminWidgetsC0Inline(NestedStackedInline):
    model = TestAdminWidgetsC0
    prepopulated_fields = {'slug': ('name', )}
    filter_horizontal = ['m2m']
    sortable_field_name = "position"
    extra = 0
    inline_classes = ("collapse", "open", "grp-collapse", "grp-open",)
    raw_id_fields = ['fk2']
    autocomplete_lookup_fields = {'fk': ['fk2']}


class TestAdminWidgetsC1Inline(NestedStackedInline):
    model = TestAdminWidgetsC1
    prepopulated_fields = {'slug': ('name', )}
    filter_horizontal = ['m2m']
    extra = 0
    inline_classes = ("collapse", "open", "grp-collapse", "grp-open",)
    raw_id_fields = ['fk2']
    autocomplete_lookup_fields = {'fk': ['fk2']}


class TestAdminWidgetsBInline(NestedStackedInline):
    model = TestAdminWidgetsB
    inlines = [TestAdminWidgetsC0Inline, TestAdminWidgetsC1Inline]
    prepopulated_fields = {'slug': ('name', )}
    filter_horizontal = ['m2m']
    sortable_field_name = "position"
    extra = 1
    inline_classes = ("collapse", "open", "grp-collapse", "grp-open",)
    raw_id_fields = ['fk2']
    autocomplete_lookup_fields = {'fk': ['fk2']}


class TestAdminWidgetsAInline(NestedStackedInline):
    model = TestAdminWidgetsA
    inlines = [TestAdminWidgetsBInline]
    prepopulated_fields = {'slug': ('name', )}
    filter_horizontal = ['m2m']
    sortable_field_name = "position"
    extra = 1
    inline_classes = ("collapse", "open", "grp-collapse", "grp-open",)
    raw_id_fields = ['fk2']
    autocomplete_lookup_fields = {'fk': ['fk2']}


@admin.register(TestAdminWidgetsRoot)
class TestAdminWidgetsRootAdmin(NestedModelAdmin):
    inlines = [TestAdminWidgetsAInline]


admin.site.register(TestAdminWidgetsRelated1, NestedModelAdmin)
admin.site.register(TestAdminWidgetsRelated2, NestedModelAdmin)
admin.site.register(TestAdminWidgetsM2M, NestedModelAdmin)



from django.shortcuts import redirect, render_to_response
from django.template import RequestContext

from forms import PhotoForm, PostForm


def generic_completion_view(request, form_class, template):
    form = form_class(request.POST or None, request.FILES or None)
    
    if request.method == 'POST' and form.is_valid():
        # save the new object instance
        new_obj = form.save()
        
        # grab the related objects from the form and add them
        # to the new post instance
        for obj in form.cleaned_data['hidden_relationships']:
            new_obj.related.connect(obj)
        
        return redirect(new_obj)
    
    return render_to_response(template, {'form': form},
        context_instance=RequestContext(request))

def create_photo(request):
    return generic_completion_view(request, PhotoForm, 'media/create_photo.html')

def create_post(request):
    return generic_completion_view(request, PostForm, 'blog/create_post.html')



from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Root(models.Model):
    slug = models.SlugField(max_length=10, blank=True, null=True)

    def __str__(self):
        return "%s(%s)" % (type(self).__name__, self.slug)


@python_2_unicode_compatible
class A(models.Model):
    root = models.ForeignKey(Root, related_name="a_set", on_delete=models.CASCADE)
    position = models.PositiveIntegerField()

    foo = models.CharField(max_length=10, blank=True, null=True)
    a_type = models.CharField(max_length=1, choices=(('X', 'X'), ('Y', 'Y')))
    default_a_type = None

    class Meta:
        ordering = ['position']

    def __str__(self):
        parts = ["A%s[%d](%s)" % (self.a_type, self.position, self.foo)]
        if self.root:
            parts.insert(0, "%s" % self.root)
        return "/".join(parts)

    def save(self, **kwargs):
        self.a_type = self.default_a_type
        super(A, self).save(**kwargs)


class AManager(models.Manager):
    def __init__(self, a_type):
        self.a_type = a_type
        super(AManager, self).__init__()

    def get_queryset(self):
        return super(AManager, self).get_queryset().filter(a_type=self.a_type)


class AX(A):
    default_a_type = 'X'
    objects = AManager(default_a_type)

    class Meta:
        proxy = True


class AY(A):
    default_a_type = 'Y'
    objects = AManager(default_a_type)

    class Meta:
        proxy = True


@python_2_unicode_compatible
class B(models.Model):
    a = models.ForeignKey(A, related_name="b_set", on_delete=models.CASCADE)
    position = models.PositiveIntegerField()

    bar = models.CharField(max_length=10, blank=True, null=True)
    b_type = models.CharField(max_length=1, choices=(('X', 'X'), ('Y', 'Y')))
    default_b_type = None

    class Meta:
        ordering = ['position']

    def __str__(self):
        parts = ["B%s[%d](%s)" % (self.b_type, self.position, self.bar)]
        if self.a:
            parts.insert(0, "%s" % self.a)
        return "/".join(parts)

    def save(self, **kwargs):
        self.b_type = self.default_b_type
        super(B, self).save(**kwargs)


class BManager(models.Manager):
    def __init__(self, b_type):
        self.b_type = b_type
        super(BManager, self).__init__()

    def get_queryset(self):
        return super(BManager, self).get_queryset().filter(b_type=self.b_type)


class BX(B):
    default_b_type = 'X'
    objects = BManager(default_b_type)

    class Meta:
        proxy = True


class BY(B):
    default_b_type = 'Y'
    objects = BManager(default_b_type)

    class Meta:
        proxy = True



import os
from setuptools import setup, find_packages

from genericm2m import VERSION

f = open(os.path.join(os.path.dirname(__file__), 'README.rst'))
readme = f.read()
f.close()

setup(
    name='django-generic-m2m',
    version=".".join(map(str, VERSION)),
    description='relate anything to anything',
    long_description=readme,
    author='Charles Leifer',
    author_email='coleifer@gmail.com',
    url='https://github.com/coleifer/django-generic-m2m',
    packages=find_packages(),
    package_data = {
        'genericm2m': [
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    test_suite='runtests.runtests',
)



from __future__ import unicode_literals

from collections import defaultdict
import functools
from itertools import dropwhile
from operator import __not__, attrgetter, eq, ge, gt, le, lt, mul

import django
from django.core.exceptions import (FieldError, MultipleObjectsReturned,
                                    ObjectDoesNotExist)
from django.db import transaction
from django.db.models.base import Model
from django.db.models.constants import LOOKUP_SEP
from django.db.models.query import EmptyQuerySet, QuerySet
from django.utils import six

# Only export the public API for QuerySetSequence. (Note that QuerySequence and
# QuerySetSequenceModel are considered semi-public: the APIs probably won't
# change, but implementation is not guaranteed. Other functions/classes are
# considered implementation details.)
__all__ = ['QuerySetSequence']


def cmp(a, b):
    """Python 2 & 3 version of cmp built-in."""
    return (a > b) - (a < b)


def multiply_iterables(it1, it2):
    """
    Element-wise iterables multiplications.
    """
    assert len(it1) == len(it2),\
        "Can not element-wise multiply iterables of different length."
    return list(map(mul, it1, it2))


def cumsum(seq):
    s = 0
    for c in seq:
        s += c
        yield s


class ComparatorMixin(object):
    @classmethod
    def _get_field_names(cls, model):
        """Return a list of field names that are part of a model."""
        return [f.name for f in model._meta.get_fields()]

    @classmethod
    def _cmp(cls, value1, value2):
        """
        Comparison method that takes into account Django's special rules when
        ordering by a field that is a model:

            1. Try following the default ordering on the related model.
            2. Order by the model's primary key, if there is no Meta.ordering.

        """
        if isinstance(value1, Model) and isinstance(value2, Model):
            field_names = value1._meta.ordering

            # Assert that the ordering is the same between different models.
            if field_names != value2._meta.ordering:
                valid_field_names = (set(cls._get_field_names(value1)) &
                                     set(cls._get_field_names(value2)))
                raise FieldError(
                    "Ordering differs between models. Choices are: %s" %
                    ', '.join(valid_field_names))

            # By default, order by the pk.
            if not field_names:
                field_names = ['pk']

            # TODO Figure out if we don't need to generate this comparator every
            # time.
            return cls._generate_comparator(field_names)(value1, value2)

        return cmp(value1, value2)

    @classmethod
    def _generate_comparator(cls, field_names):
        """
        Construct a comparator function based on the field names. The comparator
        returns the first non-zero comparison value.

        Inputs:
            field_names (iterable of strings): The field names to sort on.

        Returns:
            A comparator function.
        """
        # Ensure that field names is a list and not a tuple.
        field_names = list(field_names)

        # For fields that start with a '-', reverse the ordering of the
        # comparison.
        reverses = [1] * len(field_names)
        for i, field_name in enumerate(field_names):
            if field_name[0] == '-':
                reverses[i] = -1
                field_names[i] = field_name[1:]

        field_names = [f.replace(LOOKUP_SEP, '.') for f in field_names]

        def comparator(i1, i2):
            # Get a tuple of values for comparison.
            v1 = attrgetter(*field_names)(i1)
            v2 = attrgetter(*field_names)(i2)

            # If there's only one arg supplied, attrgetter returns a single
            # item, directly return the result in this case.
            if len(field_names) == 1:
                return cls._cmp(v1, v2) * reverses[0]

            # Compare each field for the two items, reversing if necessary.
            order = multiply_iterables(list(map(cls._cmp, v1, v2)), reverses)

            try:
                # The first non-zero element.
                return next(dropwhile(__not__, order))
            except StopIteration:
                # Everything was equivalent.
                return 0

        return comparator


class QuerySequenceIterable(ComparatorMixin):
    def __init__(self, querysetsequence):
        # Create a clone so that subsequent calls to iterate are kept separate.
        self._querysets = querysetsequence._querysets
        self._queryset_idxs = querysetsequence._queryset_idxs
        self._order_by = querysetsequence._order_by
        self._standard_ordering = querysetsequence._standard_ordering
        self._low_mark = querysetsequence._low_mark
        self._high_mark = querysetsequence._high_mark

    def _ordered_iterator(self):
        """
        Interleave the values of each QuerySet in order to handle the requested
        ordering. Also adds the '#' property to each returned item.
        """
        # A list of tuples, each with:
        #   * The iterable
        #   * The QuerySet number
        #   * The next value
        #
        # (Remember that each QuerySet is already sorted.)
        iterables = []
        for i, qs in zip(self._queryset_idxs, self._querysets):
            it = iter(qs)
            try:
                value = next(it)
            except StopIteration:
                # If this is already empty, just skip it.
                continue
            # Set the QuerySet number so that the comparison works properly.
            setattr(value, '#', i)
            iterables.append((it, i, value))

        # The offset of items returned.
        index = 0

        # Create a comparison function based on the requested ordering.
        _comparator = self._generate_comparator(self._order_by)
        def comparator(tuple_1, tuple_2):
            # The last element in each tuple is the actual item to compare.
            return _comparator(tuple_1[2], tuple_2[2])
        comparator = functools.cmp_to_key(comparator)

        # If in reverse mode, get the last value instead of the first value from
        # ordered_values below.
        if self._standard_ordering:
            next_value_ind = 0
        else:
            next_value_ind = -1

        # Continue until all iterables are empty.
        while iterables:
            # If there's only one iterator left, don't bother sorting.
            if len(iterables) > 1:
                # Sort the current values for each iterable.
                iterables = sorted(iterables, key=comparator)

                # The next ordering item is in the first position, unless we're
                # in reverse mode.
                it, i, value = iterables[next_value_ind]
            else:
                it, i, value = iterables[0]

            # Return the next value if we're within the slice of interest.
            if self._low_mark <= index:
                yield value
            index += 1
            # We've left the slice of interest, we're done.
            if index == self._high_mark:
                return

            # Iterate the iterable that just lost a value.
            try:
                value = next(it)
                # Set the QuerySet number so that the comparison works properly.
                setattr(value, '#', i)
                iterables[next_value_ind] = it, i, value
            except StopIteration:
                # This iterator is done, remove it.
                del iterables[next_value_ind]

    def _unordered_iterator(self):
        """
        Return the value of each QuerySet, but also add the '#' property to each
        return item.
        """
        for i, qs in zip(self._queryset_idxs, self._querysets):
            for item in qs:
                setattr(item, '#', i)
                yield item

    def __iter__(self):
        # If there's no QuerySets, just return an empty iterator.
        if not len(self._querysets):
            return iter([])

        # If order is necessary, evaluate and start feeding data back.
        if self._order_by:
            # If the first element of order_by is '#', this means first order by
            # QuerySet. If it isn't this, then returned the interleaved
            # iterator.
            if self._order_by[0].lstrip('-') != '#':
                return self._ordered_iterator()

            # Otherwise, order by QuerySet first. Handle reversing the
            # QuerySets, if necessary.
            elif self._order_by[0].startswith('-'):
                self._querysets = self._querysets[::-1]

        # If there is no ordering, or the ordering is specific to each QuerySet,
        # evaluation can be pushed off further.

        # If there is no slicing, iterate through each QuerySet. This avoids
        # calling count() on each QuerySet.
        if self._low_mark == 0 and self._high_mark is None:
            return self._unordered_iterator()

        # First trim any QuerySets based on the currently set limits!
        counts = [0]
        counts.extend(cumsum([it.count() for it in self._querysets]))

        # Trim the beginning of the QuerySets, if necessary.
        start_index = 0
        low_mark, high_mark = self._low_mark, self._high_mark
        if low_mark is not 0:
            # Convert a negative index into a positive.
            if low_mark < 0:
                low_mark += counts[-1]

            # Find the point when low_mark crosses a threshold.
            for i, offset in enumerate(counts):
                if offset <= low_mark:
                    start_index = i
                if low_mark < offset:
                    break

        # Trim the end of the QuerySets, if necessary.
        end_index = len(self._querysets)
        if high_mark is None:
            # If it was unset (meaning all), set it to the maximum.
            high_mark = counts[-1]
        elif high_mark:
            # Convert a negative index into a positive.
            if high_mark < 0:
                high_mark += counts[-1]

            # Find the point when high_mark crosses a threshold.
            for i, offset in enumerate(counts):
                if high_mark <= offset:
                    end_index = i
                    break

        # Remove QuerySets we don't care about.
        self._querysets = self._querysets[start_index:end_index]
        self._queryset_idxs = self._queryset_idxs[start_index:end_index]

        # The low_mark needs the removed QuerySets subtracted from it.
        low_mark -= counts[start_index]
        # The high_mark needs the count of all QuerySets before it subtracted
        # from it.
        high_mark -= counts[end_index - 1]

        # Apply the offsets to the edge QuerySets (apply the high mark first
        # in-case there's only a single QuerySet left).
        self._querysets[-1] = self._querysets[-1][:high_mark]
        self._querysets[0] = self._querysets[0][low_mark:]

        # For anything left, just iterate through each QuerySet.
        return self._unordered_iterator()


class QuerySetSequence(ComparatorMixin):
    """
    Wrapper for multiple QuerySets without the restriction on the identity of
    the base models.

    """

    def __init__(self, *args):
        self._set_querysets(args)
        # Some information necessary for properly iterating through a QuerySet.
        self._order_by = []
        self._standard_ordering = True
        self._low_mark, self._high_mark = 0, None

        self._iterable_class = QuerySequenceIterable
        self._result_cache = None

    def _set_querysets(self, querysets):
        self._querysets = list(querysets)
        # The original ordering of the QuerySets.
        self._queryset_idxs = list(range(len(self._querysets)))

    def _clone(self):
        clone = QuerySetSequence(*[qs._clone() for qs in self._querysets])
        clone._queryset_idxs = self._queryset_idxs
        clone._order_by = self._order_by
        clone._standard_ordering = self._standard_ordering
        clone._low_mark = self._low_mark
        clone._high_mark = self._high_mark

        return clone

    def _fetch_all(self):
        if self._result_cache is None:
            self._result_cache = list(self._iterable_class(self))

    # Python magic methods.

    def __len__(self):
        self._fetch_all()
        return len(self._result_cache)

    def __iter__(self):
        self._fetch_all()
        return iter(self._result_cache)

    def __bool__(self):
        self._fetch_all()
        return bool(self._result_cache)

    def __nonzero__(self):      # Python 2 compatibility
        return type(self).__bool__(self)

    def __getitem__(self, k):
        """
        Retrieves an item or slice from the set of results.
        """
        if not isinstance(k, (slice,) + six.integer_types):
            raise TypeError
        assert ((not isinstance(k, slice) and (k >= 0)) or
                (isinstance(k, slice) and (k.start is None or k.start >= 0) and
                 (k.stop is None or k.stop >= 0))), \
            "Negative indexing is not supported."

        if isinstance(k, slice):
            qs = self._clone()
            # If start is not given, it is 0.
            if k.start is not None:
                start = int(k.start)
            else:
                start = 0
            # Apply the new start to any previous slices.
            qs._low_mark += start

            # If stop is not given, don't modify the stop.
            if k.stop is not None:
                stop = int(k.stop)

                # The high mark needs to take into an account any previous
                # offsets of the low mark.
                offset = stop - start
                qs._high_mark = qs._low_mark + offset
            return list(qs)[::k.step] if k.step else qs

        qs = self._clone()
        qs._low_mark += k
        qs._high_mark = qs._low_mark + 1
        return list(qs)[0]

    def __and__(self, other):
        # If the other QuerySet is an EmptyQuerySet, this is a no-op.
        if isinstance(other, EmptyQuerySet):
            return other
        combined = self._clone()

        querysets = []
        for qs in combined._querysets:
            # Only QuerySets of the same type can have any overlap.
            if issubclass(qs.model, other.model):
                querysets.append(qs & other)

        # If none are left, we're left with an EmptyQuerySet.
        if not querysets:
            return other.none()

        combined._set_querysets(querysets)
        return combined

    def __or__(self, other):
        # If the other QuerySet is an EmptyQuerySet, this is a no-op.
        if isinstance(other, EmptyQuerySet):
            return self
        combined = self._clone()

        # If the other instance is a QuerySetSequence, combine the QuerySets.
        if isinstance(other, QuerySetSequence):
            combined._set_querysets(self._querysets + other._querysets)

        elif isinstance(other, QuerySet):
            combined._set_querysets(self._querysets + [other])

        return combined

    def _separate_filter_fields(self, **kwargs):
        qss_fields, std_fields = self._separate_fields(*kwargs.keys())

        # Remove any fields that start with '#' from kwargs.
        qss_kwargs = {field: value for field, value in kwargs.items() if field in qss_fields}
        std_kwargs = {field: value for field, value in kwargs.items() if field in std_fields}

        return qss_kwargs, std_kwargs

    def _separate_fields(self, *fields):
        # Remove any fields that start with '#' from kwargs.
        qss_fields = []
        std_fields = []
        for field in fields:
            if field.startswith('#') or field.startswith('-#'):
                qss_fields.append(field)
            else:
                std_fields.append(field)

        return qss_fields, std_fields

    def _filter_or_exclude_querysets(self, negate, **kwargs):
        """
        Similar to QuerySet._filter_or_exclude, but run over the QuerySets in
        the QuerySetSequence instead of over each QuerySet's fields.
        """
        # Ensure negate is a boolean.
        negate = bool(negate)

        for kwarg, value in kwargs.items():
            parts = kwarg.split(LOOKUP_SEP)

            # Ensure this is being used to filter QuerySets.
            if parts[0] != '#':
                raise ValueError("Keyword '%s' is not a valid keyword to filter over, "
                                 "it must begin with '#'." % kwarg)

            # Don't allow __ multiple times.
            if len(parts) > 2:
                raise ValueError("Keyword '%s' must not contain multiple "
                                 "lookup seperators." % kwarg)

            # The actual lookup is the second part.
            try:
                lookup = parts[1]
            except IndexError:
                lookup = 'exact'

            # Math operators that all have the same logic.
            LOOKUP_TO_OPERATOR = {
                'exact': eq,
                'iexact': eq,
                'gt': gt,
                'gte': ge,
                'lt': lt,
                'lte': le,
            }
            try:
                operator = LOOKUP_TO_OPERATOR[lookup]

                # These expect integers, this matches the logic in
                # IntegerField.get_prep_value(). (Essentially treat the '#'
                # field as an IntegerField.)
                if value is not None:
                    value = int(value)

                self._queryset_idxs = filter(lambda i: operator(i, value) != negate, self._queryset_idxs)
                continue
            except KeyError:
                # It wasn't one of the above operators, keep trying.
                pass

            # Some of these seem to get handled as bytes.
            if lookup in ('contains', 'icontains'):
                value = six.text_type(value)
                self._queryset_idxs = filter(lambda i: (value in six.text_type(i)) != negate, self._queryset_idxs)

            elif lookup == 'in':
                self._queryset_idxs = filter(lambda i: (i in value) != negate, self._queryset_idxs)

            elif lookup in ('startswith', 'istartswith'):
                value = six.text_type(value)
                self._queryset_idxs = filter(lambda i: six.text_type(i).startswith(value) != negate, self._queryset_idxs)

            elif lookup in ('endswith', 'iendswith'):
                value = six.text_type(value)
                self._queryset_idxs = filter(lambda i: six.text_type(i).endswith(value) != negate, self._queryset_idxs)

            elif lookup == 'range':
                # Inclusive include.
                start, end = value
                self._queryset_idxs = filter(lambda i: (start <= i <= end) != negate, self._queryset_idxs)

            else:
                # Any other field lookup is not supported, e.g. date, year, month,
                # day, week_day, hour, minute, second, isnull, search, regex, and
                # iregex.
                raise ValueError("Unsupported lookup '%s'" % lookup)

        # Convert back to a list on Python 3.
        self._queryset_idxs = list(self._queryset_idxs)

        # Finally, keep only the QuerySets we care about!
        self._querysets = [self._querysets[i] for i in self._queryset_idxs]

    # Methods that return new QuerySets
    def filter(self, **kwargs):
        qss_fields, fields = self._separate_filter_fields(**kwargs)

        clone = self._clone()
        clone._filter_or_exclude_querysets(False, **qss_fields)
        clone._querysets = [qs.filter(**fields) for qs in clone._querysets]
        return clone

    def exclude(self, **kwargs):
        qss_fields, fields = self._separate_filter_fields(**kwargs)

        clone = self._clone()
        clone._filter_or_exclude_querysets(True, **qss_fields)
        clone._querysets = [qs.exclude(**fields) for qs in clone._querysets]
        return clone

    def annotate(self, *args, **kwargs):
        clone = self._clone()
        clone._querysets = [qs.annotate(*args, **kwargs) for qs in clone._querysets]
        return clone

    def order_by(self, *fields):
        _, filtered_fields = self._separate_fields(*fields)

        # Apply the filtered fields to each underlying QuerySet.
        clone = self._clone()
        clone._querysets = [qs.order_by(*filtered_fields) for qs in self._querysets]

        # But keep the original fields for the clone.
        clone._order_by = list(fields)
        return clone

    def reverse(self):
        clone = self._clone()
        clone._querysets = [qs.reverse() for qs in reversed(self._querysets)]
        clone._standard_ordering = not self._standard_ordering
        return clone

    def distinct(self, *fields):
        raise NotImplementedError()

    def values(self, *fields, **expressions):
        raise NotImplementedError()

    def values_list(self, *fields, **kwargs):
        raise NotImplementedError()

    def dates(self, field, kind, order='ASC'):
        raise NotImplementedError()

    def datetimes(self, field_name, kind, order='ASC', tzinfo=None):
        raise NotImplementedError()

    def none(self):
        # This is a bit odd, but use the first QuerySet to properly return an
        # that is an instance of EmptyQuerySet.
        return self._querysets[0].none()

    def all(self):
        clone = self._clone()
        clone._querysets = [qs.all() for qs in self._querysets]
        return clone

    def union(self, *other_qs, **kwargs):
        raise NotImplementedError()

    def intersection(self, *other_qs, **kwargs):
        raise NotImplementedError()

    def difference(self, *other_qs, **kwargs):
        raise NotImplementedError()

    def select_related(self, *fields):
        clone = self._clone()
        clone._querysets = [qs.select_related(*fields) for qs in self._querysets]
        return clone

    def prefetch_related(self, *lookups):
        clone = self._clone()
        clone._querysets = [qs.prefetch_related(*lookups) for qs in self._querysets]
        return clone

    def extra(self, select=None, where=None, params=None, tables=None, order_by=None, select_params=None):
        clone = self._clone()
        clone._querysets = [qs.extra(select=select, where=where, params=params, tables=tables, order_by=order_by, select_params=select_params) for qs in self._querysets]
        return clone

    def defer(self, *fields):
        clone = self._clone()
        clone._querysets = [qs.defer(*fields) for qs in self._querysets]
        return clone

    def only(self, *fields):
        clone = self._clone()
        clone._querysets = [qs.only(*fields) for qs in self._querysets]
        return clone

    def using(self, alias):
        clone = self._clone()
        clone._querysets = [qs.using(alias) for qs in self._querysets]
        return clone

    def select_for_update(self):
        raise NotImplementedError()

    def raw(self):
        raise NotImplementedError()

    # Methods that do not return QuerySets
    def get(self, **kwargs):
        clone = self.filter(**kwargs)

        result = None
        for qs in clone._querysets:
            try:
                obj = qs.get()
            except ObjectDoesNotExist:
                pass
            # Don't catch the MultipleObjectsReturned(), allow it to raise.
            else:
                # If a second object is found, raise an exception.
                if result:
                    raise MultipleObjectsReturned()
                result = obj

        # Checked all QuerySets and no object was found.
        if result is None:
            raise ObjectDoesNotExist()

        # Return the only result found.
        return result

    def create(self, **kwargs):
        raise NotImplementedError()

    def get_or_create(self, defaults=None, **kwargs):
        raise NotImplementedError()

    def update_or_create(self, defaults=None, **kwargs):
        raise NotImplementedError()

    def bulk_create(self, objs, batch_size=None, ignore_conflicts=False):
        raise NotImplementedError()

    def count(self):
        return sum(qs.count() for qs in self._querysets) - self._low_mark

    def in_bulk(self, id_list=None, field_name='pk'):
        raise NotImplementedError()

    def iterator(self):
        clone = self._clone()
        clone._querysets = [qs.iterator() for qs in self._querysets]
        return clone

    def _get_latest_by(self):
        """Process get_latest_by Meta on each QuerySet and return the value."""
        # Get each QuerySet's get_latest_by (ignore unset values).
        get_latest_by = map(lambda qs: getattr(qs.model._meta, 'get_latest_by'), self._querysets)
        get_latest_by = set(get_latest_by)

        # Ensure all of them are identical.
        if len(get_latest_by) > 1:
            raise ValueError(
                "earliest() and latest() require 'get_latest_by' in each "
                "model's Meta to be identical.")

        # If all the values are None, get_latest_by was not set.
        if not get_latest_by:
            raise ValueError(
                "earliest() and latest() require either fields as positional "
                "arguments or 'get_latest_by' in the model's Meta.")

        # Cast to a list and return the value.
        return list(get_latest_by)

    def _get_first_or_last(self, items, order_fields, reverse):
        # Generate a comparator and sort the items.
        comparator = self._generate_comparator(order_fields)
        items = sorted(items, key=functools.cmp_to_key(comparator), reverse=reverse)

        # Return the first one (whether this is first or last is controlled by
        # reverse).
        return items[0]

    def latest(self, *fields):
        # If fields are given, fallback to get_latest_by.
        if not fields:
            fields = self._get_latest_by()

        objs = []
        for qs in self._querysets:
            try:
                objs.append(qs.latest(*fields))
            except ObjectDoesNotExist:
                pass

        # Checked all QuerySets and no object was found.
        if not objs:
            raise ObjectDoesNotExist()

        # Return the latest.
        return self._get_first_or_last(objs, fields, True)

    def earliest(self, *fields):
        # If fields are given, fallback to get_latest_by.
        if not fields:
            fields = self._get_latest_by()

        objs = []
        for qs in self._querysets:
            try:
                objs.append(qs.earliest(*fields))
            except ObjectDoesNotExist:
                pass

        # Checked all QuerySets and no object was found.
        if not objs:
            raise ObjectDoesNotExist()

        # Return the latest.
        return self._get_first_or_last(objs, fields, False)

    def first(self):
        # If there's no QuerySets, return None. If the QuerySets are unordered,
        # use the first item of first QuerySet. If ordered, compare the first
        # item of each QuerySet to find the overall first.
        if not self._querysets:
            return None

        elif not self.ordered:
            return self._querysets[0].first()

        else:
            # Get each first item for each and compare them, return the "first".
            return self._get_first_or_last(
                [qs.first() for qs in self._querysets], self._order_by, False)

    def last(self):
        # See the comments for first().
        if not self._querysets:
            return None

        elif not self.ordered:
            return self._querysets[-1].last()

        else:
            # Get each last item for each and compare them, return the "last".
            return self._get_first_or_last(
                [qs.last() for qs in self._querysets], self._order_by, True)

    def aggregate(self, *args, **kwargs):
        raise NotImplementedError()

    def exists(self):
        return any(qs.exists() for qs in self._querysets)

    def update(self, **kwargs):
        with transaction.atomic():
            return sum(qs.update(**kwargs) for qs in self._querysets)

    def delete(self):
        deleted_count = 0
        deleted_objects = defaultdict(int)
        for qs in self._querysets:
            # Delete this QuerySet.
            current_deleted_count, current_deleted_objects = qs.delete()

            # Combine the results.
            deleted_count += current_deleted_count
            for obj, count in current_deleted_objects.items():
                deleted_objects[obj] += count

        return deleted_count, dict(deleted_objects)

    def as_manager(self):
        raise NotImplementedError()

    if django.VERSION >= (2, 1):
        def explain(self, format=None, **options):
            return '\n'.join(qs.explain() for qs in self._querysets)

    # Public attributes
    @property
    def ordered(self):
        """
        Returns True if the QuerySet is ordered -- i.e. has an order_by()
        clause.
        """
        return bool(self._order_by)

    # Methods specific to QuerySetSequence.
    def get_querysets(self):
        """Returns a list of the QuerySet objects which form the sequence."""
        return self._querysets



from django.contrib import admin
from django import forms
import nested_admin

from .models import Root, A, AX, AY, B, BX, BY


class BXModelForm(forms.ModelForm):
    class Meta:
        model = BX
        exclude = ['b_type']


class BYModelForm(forms.ModelForm):
    class Meta:
        model = BY
        exclude = ['b_type']


class BInline(nested_admin.NestedStackedInline):
    model = B
    extra = 0
    sortable_field_name = "position"
    inline_classes = ("collapse", "open", )

    def get_queryset(self, request):
        qset = super(BInline, self).get_queryset(request)
        return qset.filter(b_type=self.model.default_b_type)


class BXInline(BInline):
    model = BX
    form = BXModelForm


class BYInline(BInline):
    model = BY
    form = BYModelForm
    extra = 0


class AXModelForm(forms.ModelForm):
    class Meta:
        model = AX
        exclude = ['a_type']


class AYModelForm(forms.ModelForm):
    class Meta:
        model = AY
        exclude = ['a_type']


class AInline(nested_admin.NestedStackedInline):
    model = A
    extra = 0
    inlines = [BXInline, BYInline]
    sortable_field_name = "position"
    inline_classes = ("collapse", "open", )

    def get_queryset(self, request):
        qset = super(AInline, self).get_queryset(request)
        return qset.filter(a_type=self.model.default_a_type)


class AXInline(AInline):
    model = AX
    form = AXModelForm


class AYInline(AInline):
    model = AY
    form = AYModelForm
    extra = 0


@admin.register(Root)
class RootAdmin(nested_admin.NestedModelAdmin):

    inlines = [AXInline, AYInline]



from django.conf import settings
from django.conf.urls.defaults import *
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^autocomplete/', include('completion.urls')),
    url(r'^blog/', include('basic.blog.urls')),
    url(r'^create/', include('example.site_app.urls')), # our custom content creation views
    url(r'^media/', include('basic.media.urls.photos')),
    url(r'^people/', include('basic.people.urls')),
    url(r'^places/', include('basic.places.urls')),
    url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {
        'document_root': settings.MEDIA_ROOT,
    }),
    url(r'^$', 'django.views.generic.simple.direct_to_template', kwargs={'template': 'homepage.html'}),
)



try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models

from genericm2m.models import RelatedObjectsDescriptor


class RelatedBeverage(models.Model):
    food = models.ForeignKey('Food')
    beverage = models.ForeignKey('Beverage')

    class Meta:
        ordering = ('-id',)


class Food(models.Model):
    name = models.CharField(max_length=255)

    related = RelatedObjectsDescriptor()
    related_beverages = RelatedObjectsDescriptor(RelatedBeverage, 'food', 'beverage')

    def __unicode__(self):
        return self.name


class Beverage(models.Model):
    name = models.CharField(max_length=255)

    related = RelatedObjectsDescriptor()

    def __unicode__(self):
        return self.name


class Person(models.Model):
    name = models.CharField(max_length=255)

    related = RelatedObjectsDescriptor()

    def __unicode__(self):
        return self.name


class Boring(models.Model):
    name = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name


class AnotherRelatedObject(models.Model):
    parent_type = models.ForeignKey(ContentType, related_name="child_%(class)s")
    parent_id = models.IntegerField(db_index=True)
    parent = GenericForeignKey(ct_field="parent_type", fk_field="parent_id")

    object_type = models.ForeignKey(ContentType, related_name="related_%(class)s")
    object_id = models.IntegerField(db_index=True)
    object = GenericForeignKey(ct_field="object_type", fk_field="object_id")

    alias = models.CharField(max_length=255, blank=True)
    description = models.TextField(blank=True)

    creation_date = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ('id',)


class Note(models.Model):
    content = models.TextField()

    related = RelatedObjectsDescriptor(AnotherRelatedObject)



from django.db import models


class Author(models.Model):
    name = models.CharField(max_length=50)

    class Meta:
        ordering = ['name']

    def __str__(self):
        return self.name


class Publisher(models.Model):
    name = models.CharField(max_length=50)
    address = models.CharField(max_length=50)


class PeriodicalPublisher(models.Model):
    # Stupid distinction, but I need a similar object with a missing field.
    name = models.CharField(max_length=50)


class OnlinePublisher(models.Model):
    # Stupid distinction, but same as PeriodicalPublisher with an ordering.
    name = models.CharField(max_length=50)

    class Meta:
        ordering = ['name']


class Article(models.Model):
    title = models.CharField(max_length=100)
    author = models.ForeignKey(Author, on_delete=models.CASCADE)
    publisher = models.ForeignKey(PeriodicalPublisher, related_name='published', on_delete=models.CASCADE)
    release = models.DateField()

    class Meta:
        get_latest_by = 'release'

    def __str__(self):
        return "%s by %s" % (self.title, self.author)


class BlogPost(models.Model):
    title = models.CharField(max_length=100)
    author = models.ForeignKey(Author, on_delete=models.CASCADE)
    publisher = models.ForeignKey(OnlinePublisher, related_name='published', on_delete=models.CASCADE)


class Book(models.Model):
    title = models.CharField(max_length=50)
    author = models.ForeignKey(Author, on_delete=models.CASCADE)
    publisher = models.ForeignKey(Publisher, related_name='published', on_delete=models.CASCADE)
    release = models.DateField()
    pages = models.PositiveSmallIntegerField()

    class Meta:
        get_latest_by = 'release'

    def __str__(self):
        return "%s by %s" % (self.title, self.author)



from __future__ import unicode_literals

from django.db import models
from django.db.models import ForeignKey, CASCADE
from django.utils.encoding import python_2_unicode_compatible


class TestAdminWidgetsRoot(models.Model):
    name = models.CharField(max_length=200)


@python_2_unicode_compatible
class TestAdminWidgetsRelated1(models.Model):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class TestAdminWidgetsRelated2(models.Model):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name

    @staticmethod
    def autocomplete_search_fields():
        return ("name__icontains", )


@python_2_unicode_compatible
class TestAdminWidgetsM2M(models.Model):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name


class TestAdminWidgetsA(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField()
    parent = ForeignKey(TestAdminWidgetsRoot, on_delete=CASCADE)
    position = models.PositiveIntegerField()
    date = models.DateTimeField(blank=True, null=True)
    upload = models.FileField(blank=True, null=True, upload_to='foo')
    fk1 = models.ForeignKey(TestAdminWidgetsRelated1, blank=True, null=True,
        on_delete=CASCADE, related_name='+')
    fk2 = models.ForeignKey(TestAdminWidgetsRelated1, blank=True, null=True,
        on_delete=CASCADE, related_name='+')
    m2m = models.ManyToManyField(TestAdminWidgetsM2M, blank=True)

    class Meta:
        ordering = ('position', )


class TestAdminWidgetsB(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField()
    parent = ForeignKey(TestAdminWidgetsA, on_delete=CASCADE)
    position = models.PositiveIntegerField()
    date = models.DateTimeField(blank=True, null=True)
    upload = models.FileField(blank=True, null=True, upload_to='foo')
    fk1 = models.ForeignKey(TestAdminWidgetsRelated1, blank=True, null=True,
        on_delete=CASCADE, related_name='+')
    fk2 = models.ForeignKey(TestAdminWidgetsRelated1, blank=True, null=True,
        on_delete=CASCADE, related_name='+')
    m2m = models.ManyToManyField(TestAdminWidgetsM2M, blank=True)

    class Meta:
        ordering = ('position', )


class TestAdminWidgetsC0(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField()
    parent = ForeignKey(TestAdminWidgetsB, on_delete=CASCADE)
    position = models.PositiveIntegerField()
    date = models.DateTimeField(blank=True, null=True)
    upload = models.FileField(blank=True, null=True, upload_to='foo')
    fk1 = models.ForeignKey(TestAdminWidgetsRelated1, blank=True, null=True,
        on_delete=CASCADE, related_name='+')
    fk2 = models.ForeignKey(TestAdminWidgetsRelated1, blank=True, null=True,
        on_delete=CASCADE, related_name='+')
    m2m = models.ManyToManyField(TestAdminWidgetsM2M, blank=True)

    class Meta:
        ordering = ('position', )


class TestAdminWidgetsC1(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField()
    parent = ForeignKey(TestAdminWidgetsB, on_delete=CASCADE)
    position = models.PositiveIntegerField()
    date = models.DateTimeField(blank=True, null=True)
    upload = models.FileField(blank=True, null=True, upload_to='foo')
    fk1 = models.ForeignKey(TestAdminWidgetsRelated1, blank=True, null=True,
        on_delete=CASCADE, related_name='+')
    fk2 = models.ForeignKey(TestAdminWidgetsRelated1, blank=True, null=True,
        on_delete=CASCADE, related_name='+')
    m2m = models.ManyToManyField(TestAdminWidgetsM2M, blank=True)

    class Meta:
        ordering = ('position', )



"""Autocomplete fields for django-queryset-sequence and django-generic-m2m."""

from dal_genericm2m.fields import GenericM2MFieldMixin

from dal_queryset_sequence.fields import QuerySetSequenceModelMultipleField


class GenericM2MQuerySetSequenceField(GenericM2MFieldMixin,
                                      QuerySetSequenceModelMultipleField):
    """Autocomplete field for GM2MField() for QuerySetSequence choices."""



from __future__ import unicode_literals

from django.db import models
from django.db.models import ForeignKey, CASCADE


class TopLevel(models.Model):
    name = models.CharField(max_length=200)


class LevelOne(models.Model):
    name = models.CharField(max_length=200)
    parent_level = ForeignKey(TopLevel, related_name='children', on_delete=CASCADE)
    position = models.PositiveIntegerField()

    class Meta:
        ordering = ('position', )


class LevelTwo(models.Model):
    name = models.CharField(max_length=200, blank=True)
    parent_level = ForeignKey(LevelOne, related_name='children', on_delete=CASCADE)
    position = models.PositiveIntegerField(blank=True, null=True)

    class Meta:
        ordering = ('position', )


class LevelThree(models.Model):
    name = models.CharField(max_length=200)
    parent_level = ForeignKey(LevelTwo, related_name='children', on_delete=CASCADE)
    position = models.PositiveIntegerField()

    class Meta:
        ordering = ('position', )



from django.contrib import admin
from nested_admin import (
    NestedStackedInline, NestedTabularInline, NestedModelAdmin)

from .models import (
    StackedGroup, StackedSection, StackedItem,
    TabularGroup, TabularSection, TabularItem,
    SortableWithExtraRoot, SortableWithExtraChild)


class StackedItemInline(NestedStackedInline):
    model = StackedItem
    extra = 0
    sortable_field_name = "position"
    inline_classes = ("collapse", "open", )


class StackedSectionInline(NestedStackedInline):
    model = StackedSection
    extra = 0
    sortable_field_name = "position"
    inlines = [StackedItemInline]
    inline_classes = ("collapse", "open", )


@admin.register(StackedGroup)
class StackedGroupAdmin(NestedModelAdmin):
    inlines = [StackedSectionInline]


class TabularItemInline(NestedTabularInline):
    model = TabularItem
    extra = 0
    sortable_field_name = "position"


class TabularSectionInline(NestedTabularInline):
    model = TabularSection
    extra = 0
    sortable_field_name = "position"
    inlines = [TabularItemInline]


@admin.register(TabularGroup)
class TabularGroupAdmin(NestedModelAdmin):
    inlines = [TabularSectionInline]


class SortableWithExtraChildInline(NestedStackedInline):
    model = SortableWithExtraChild
    extra = 2
    sortable_field_name = "position"
    inline_classes = ("collapse", "open", )
    sortable_excludes = ['foo']


@admin.register(SortableWithExtraRoot)
class SortableWithExtraRootAdmin(NestedModelAdmin):
    inlines = [SortableWithExtraChildInline]



from nested_admin.tests.base import BaseNestedAdminTestCase
from .models import TopLevel, LevelOne, LevelTwo, LevelThree


class TestDeepNesting(BaseNestedAdminTestCase):

    root_model = TopLevel
    nested_models = (LevelOne, LevelTwo, LevelThree)

    @classmethod
    def setUpClass(cls):
        super(TestDeepNesting, cls).setUpClass()
        cls.l1_model, cls.l2_model, cls.l3_model = cls.nested_models

    def test_validationerror_on_empty_extra_parent_form(self):
        toplevel = TopLevel.objects.create(name='a')
        self.load_admin(toplevel)

        self.set_field('name', 'c', indexes=[0, 0])
        self.set_field('name', 'd', indexes=[0, 0, 0])

        self.save_form()

        field_id_with_error = self.selenium.execute_script(
            "return $('ul.errorlist li').closest('.form-row').find('input').attr('id')")

        self.assertEqual(field_id_with_error, "id_children-0-name")

    def test_create_new(self):
        self.load_admin()
        self.set_field('name', 'a')
        self.set_field('name', 'b', [0])
        self.set_field('name', 'c', [0, 0])
        self.set_field('name', 'd', [0, 0, 0])
        self.save_form()

        root_instances = self.root_model.objects.all()
        self.assertNotEqual(len(root_instances), 0, "%s did not save" % self.root_model.__name__)
        self.assertEqual(len(root_instances), 1, "Too many %s found" % self.root_model.__name__)
        root = root_instances[0]
        self.assertEqual(root.name, 'a', "%s.name has wrong value" % self.root_model.__name__)
        l1_instances = root.children.all()
        self.assertNotEqual(len(l1_instances), 0, "%s did not save" % self.l1_model.__name__)
        self.assertEqual(len(l1_instances), 1, "Too many %s found" % self.l1_model.__name__)
        l1_instance = l1_instances[0]
        self.assertEqual(l1_instance.name, 'b', "%s.name has wrong value" % self.l1_model.__name__)
        l2_instances = l1_instance.children.all()
        self.assertNotEqual(len(l2_instances), 0, "%s did not save" % self.l2_model.__name__)
        self.assertEqual(len(l2_instances), 1, "Too many %s found" % self.l2_model.__name__)
        l2_instance = l2_instances[0]
        self.assertEqual(l2_instance.name, 'c', "%s.name has wrong value" % self.l2_model.__name__)
        l3_instances = l2_instance.children.all()
        self.assertNotEqual(len(l3_instances), 0, "%s did not save" % self.l3_model.__name__)
        self.assertEqual(len(l3_instances), 1, "Too many %s found" % self.l3_model.__name__)
        l3_instance = l3_instances[0]
        self.assertEqual(l3_instance.name, 'd', "%s.name has wrong value" % self.l3_model.__name__)

    def test_create_new_no_extras(self):
        self.load_admin()
        self.set_field('name', 'a')
        self.remove_inline([0])
        self.add_inline(name='b')
        self.remove_inline([0, 0])
        self.add_inline([0], name='c')
        self.remove_inline([0, 0, 0])
        self.add_inline([0, 0], name='d')
        self.save_form()

        root_instances = self.root_model.objects.all()
        self.assertNotEqual(len(root_instances), 0, "%s did not save" % self.root_model.__name__)
        self.assertEqual(len(root_instances), 1, "Too many %s found" % self.root_model.__name__)
        root = root_instances[0]
        self.assertEqual(root.name, 'a', "%s.name has wrong value" % self.root_model.__name__)
        l1_instances = root.children.all()
        self.assertNotEqual(len(l1_instances), 0, "%s did not save" % self.l1_model.__name__)
        self.assertEqual(len(l1_instances), 1, "Too many %s found" % self.l1_model.__name__)
        l1_instance = l1_instances[0]
        self.assertEqual(l1_instance.name, 'b', "%s.name has wrong value" % self.l1_model.__name__)
        l2_instances = l1_instance.children.all()
        self.assertNotEqual(len(l2_instances), 0, "%s did not save" % self.l2_model.__name__)
        self.assertEqual(len(l2_instances), 1, "Too many %s found" % self.l2_model.__name__)
        l2_instance = l2_instances[0]
        self.assertEqual(l2_instance.name, 'c', "%s.name has wrong value" % self.l2_model.__name__)
        l3_instances = l2_instance.children.all()
        self.assertNotEqual(len(l3_instances), 0, "%s did not save" % self.l3_model.__name__)
        self.assertEqual(len(l3_instances), 1, "Too many %s found" % self.l3_model.__name__)
        l3_instance = l3_instances[0]
        self.assertEqual(l3_instance.name, 'd', "%s.name has wrong value" % self.l3_model.__name__)

    def test_save_missing_intermediate_inline(self):
        self.load_admin()
        self.set_field('name', 'a')
        self.set_field('name', 'b', [0])
        self.set_field('name', 'd', [0, 0, 0])
        self.save_form()

        root_instances = self.root_model.objects.all()
        self.assertNotEqual(len(root_instances), 0, "%s did not save" % self.root_model.__name__)



"""GM2MField support for autocomplete fields."""


class GM2MFieldMixin(object):
    """GM2MField ror FutureModelForm."""

    def value_from_object(self, instance, name):
        """Return the list of objects in the GM2MField relation."""
        return None if not instance.pk else [
            getattr(x, 'gm2m_tgt', x)
            for x in getattr(instance, name).all()
        ]

    def save_relation_data(self, instance, name, value):
        """Save the relation into the GM2MField."""
        setattr(instance, name, value)



from __future__ import absolute_import, unicode_literals

import contextlib

import django
from django.db import models
from django.core.exceptions import ValidationError
from django.contrib.contenttypes.forms import BaseGenericInlineFormSet
from django.contrib.contenttypes.models import ContentType
from django.forms.models import BaseInlineFormSet
from django.utils.encoding import force_text
from django.utils import six
from django.utils.six.moves import xrange


@contextlib.contextmanager
def mutable_querydict(qd):
    orig_mutable = None
    if getattr(qd, '_mutable', None) is False:
        orig_mutable = False
        qd._mutable = True
    yield
    if orig_mutable is not None:
        qd._mutable = orig_mutable


class NestedInlineFormSetMixin(object):

    is_nested = False

    def _construct_form(self, i, **kwargs):
        defaults = {}
        if '-empty-' in self.prefix:
            defaults['empty_permitted'] = True
        defaults.update(kwargs)
        return super(NestedInlineFormSetMixin, self)._construct_form(i, **defaults)

    def save(self, commit=True):
        """
        Saves model instances for every form, adding and changing instances
        as necessary, and returns the list of instances.
        """
        self.changed_objects = []
        self.deleted_objects = []
        self.new_objects = []

        # Copied lines are from BaseModelFormSet.save()
        if not commit:
            self.saved_forms = []
            def save_m2m():
                for form in self.saved_forms:
                    form.save_m2m()
            self.save_m2m = save_m2m
        # End copied lines from BaseModelFormSet.save()

        # The above if clause is the entirety of BaseModelFormSet.save(),
        # along with the following return:
        # return self.save_existing_objects(commit) + self.save_new_objects(commit)

        # Iterate through self.forms and add properties `_list_position`
        # and `_is_initial` so that the forms can be put back in the
        # proper order at the end of the save method.
        #
        # We need to re-sort the forms because we can get ForeignKey
        # constraint errors if we save nested formsets in their default order.
        initial_form_count = self.initial_form_count()
        forms = []
        for i, form in enumerate(self.forms):
            form._list_position = i
            form._is_initial = bool(i < initial_form_count)
            forms.append(form)

        # Perform the sort (and allow extended logic in child classes)
        forms = self.process_forms_pre_save(forms)

        form_instances = []
        saved_instances = []

        for form in forms:
            instance = self.get_saved_instance_for_form(form, commit, form_instances)
            if instance is not None:
                # Copy _list_position to instance
                instance._list_position = form._list_position

                # Store saved instances so we can reference it for
                # sub-instanced nested beneath not-yet-saved instances.
                saved_instances += [instance]
            else:
                instance = form.instance
            if not self._should_delete_form(form):
                form_instances.append(instance)

        # Re-sort back to original order
        saved_instances.sort(key=lambda i: i._list_position)
        return saved_instances

    def process_forms_pre_save(self, forms):
        """
        Sort by the sortable_field_name of the formset, if it has been set,
        and re-index the form positions (to account for gaps caused by blank
        or deleted forms)

        Allows customizable sorting and modification of self.forms before
        they're iterated through in save().

        Returns list of forms.
        """
        sort_field = getattr(self, 'sortable_field_name', None)

        def get_position(form):
            return getattr(form, 'cleaned_data', {sort_field: 0}).get(sort_field, 0)

        if sort_field is not None:
            forms.sort(key=get_position)

            i = 0
            for form in forms:
                if self._should_delete_form(form):
                    # Skip deleted forms
                    continue

                original_position = form.data.get(form.add_prefix(sort_field))

                if ("%s" % i) != original_position:
                    # If this is an unchanged extra form, continue because
                    # this form will be skipped when saving
                    if not form.changed_data and not form._is_initial:
                        continue

                    # Set the sort field on the instance and in the form data
                    setattr(form.instance, sort_field, i)
                    with mutable_querydict(form.data):
                        form.data[form.add_prefix(sort_field)] = six.text_type(i)

                    # Force recalculation of changed_data
                    if django.VERSION > (1, 9):
                        form.__dict__.pop('changed_data', None)
                    else:
                        form._changed_data = None

                i += 1

        return forms

    def get_saved_instance_for_form(self, form, commit, form_instances=None):
        pk_name = None
        if form.instance and form.instance._meta.pk:
            pk_name = form.instance._meta.pk.name
        pk_val = None
        if not form.errors and hasattr(form, 'cleaned_data'):
            pk_val = form.cleaned_data.get(pk_name)
        # Inherited models will show up as instances of the parent in
        # cleaned_data
        if isinstance(pk_val, models.Model):
            pk_val = pk_val.pk
        if pk_val is not None:
            try:
                setattr(form.instance, pk_name, pk_val)
            except ValueError:
                pk_attname = form.instance._meta.pk.get_attname()
                setattr(form.instance, pk_attname, pk_val)

        if form._is_initial:
            instances = self.save_existing_objects([form], commit)
        else:
            instances = self.save_new_objects([form], commit)
        if len(instances):
            instance = instances[0]
            instance._list_position = form._list_position
            return instance
        else:
            return None

    def get_queryset(self):
        """
        TODO: document this extended method
        """
        if not self.data:
            return super(NestedInlineFormSetMixin, self).get_queryset()

        if not hasattr(self, '__queryset'):
            pk_keys = ["%s-%s" % (self.add_prefix(i), self.model._meta.pk.name)
                       for i in xrange(0, self.initial_form_count())]
            pk_vals = [self.data.get(pk_key) for pk_key in pk_keys if self.data.get(pk_key)]

            qs = self.model._default_manager.get_queryset()
            qs = qs.filter(pk__in=pk_vals)

            # If the queryset isn't already ordered we need to add an
            # artificial ordering here to make sure that all formsets
            # constructed from this queryset have the same form order.
            if not qs.ordered:
                qs = qs.order_by(self.model._meta.pk.name)

            self.__queryset = qs
        return self.__queryset

    def save_existing_objects(self, initial_forms=None, commit=True):
        """
        Identical to parent class, except ``self.initial_forms`` is replaced
        with ``initial_forms``, passed as parameter.
        """
        if not initial_forms:
            return []

        saved_instances = []

        forms_to_delete = self.deleted_forms

        for form in initial_forms:
            pk_name = self._pk_field.name

            if not hasattr(form, '_raw_value'):
                # Django 1.9+
                raw_pk_value = form.fields[pk_name].widget.value_from_datadict(
                    form.data, form.files, form.add_prefix(pk_name))
            else:
                raw_pk_value = form._raw_value(pk_name)

            # clean() for different types of PK fields can sometimes return
            # the model instance, and sometimes the PK. Handle either.
            if self._should_delete_form(form):
                pk_value = raw_pk_value
            else:
                try:
                    pk_value = form.fields[pk_name].clean(raw_pk_value)
                except ValidationError:
                    # The current form's instance was initially nested under
                    # a form that was deleted, which causes the pk clean to
                    # fail (because the instance has been deleted). To get
                    # around this we clear the pk and save it as if it were new.
                    with mutable_querydict(form.data):
                        form.data[form.add_prefix(pk_name)] = ''

                    if not form.has_changed():
                        if django.VERSION > (1, 9):
                            form.__dict__['changed_data'].append(pk_name)
                        else:
                            form._changed_data.append(pk_name)

                    saved_instances.extend(self.save_new_objects([form], commit))
                    continue
                pk_value = getattr(pk_value, 'pk', pk_value)

            obj = None
            if obj is None and form.instance and pk_value:
                model_cls = form.instance.__class__
                try:
                    obj = model_cls.objects.get(pk=pk_value)
                except model_cls.DoesNotExist:
                    if pk_value and force_text(form.instance.pk) == force_text(pk_value):
                        obj = form.instance
            if obj is None:
                obj = self._existing_object(pk_value)

            if form in forms_to_delete:
                self.deleted_objects.append(obj)
                if hasattr(self, 'delete_existing'):
                    self.delete_existing(obj, commit=commit)
                else:
                    if commit:
                        obj.delete()
                continue

            # fk_val: The value one should find in the form's foreign key field
            old_ct_val = ct_val = ContentType.objects.get_for_model(self.instance.__class__).pk
            old_fk_val = fk_val = self.instance.pk
            if form.instance.pk:
                original_instance = self.model.objects.get(pk=form.instance.pk)
                fk_field = getattr(self, 'fk', getattr(self, 'ct_fk_field', None))
                if fk_field:
                    old_fk_val = getattr(original_instance, fk_field.get_attname())
                ct_field = getattr(self, 'ct_field', None)
                if ct_field:
                    old_ct_val = getattr(original_instance, ct_field.get_attname())

            if form.has_changed() or fk_val != old_fk_val or ct_val != old_ct_val:
                self.changed_objects.append((obj, form.changed_data))
                saved_instances.append(self.save_existing(form, obj, commit=commit))
                if not commit:
                    self.saved_forms.append(form)
        return saved_instances

    def save_new_objects(self, extra_forms=None, commit=True):
        """
        Identical to parent class, except ``self.extra_forms`` is replaced
        with ``extra_forms``, passed as parameter, and self.new_objects is
        replaced with ``new_objects``.
        """
        new_objects = []

        if extra_forms is None:
            return new_objects

        for form in extra_forms:
            if not form.has_changed():
                continue
            # If someone has marked an add form for deletion, don't save the
            # object.
            if self.can_delete and self._should_delete_form(form):
                continue
            new_objects.append(self.save_new(form, commit=commit))
            if not commit:
                self.saved_forms.append(form)

        self.new_objects.extend(new_objects)
        return new_objects


class NestedInlineFormSet(NestedInlineFormSetMixin, BaseInlineFormSet):
    """
    The nested InlineFormSet for the common case (ForeignKey inlines)
    """
    pass


class NestedBaseGenericInlineFormSet(NestedInlineFormSetMixin, BaseGenericInlineFormSet):
    """
    The nested InlineFormSet for inlines of generic content-type relations
    """

    def save_existing(self, form, instance, commit=True):
        """Saves and returns an existing model instance for the given form."""
        setattr(form.instance, self.ct_field.get_attname(),
            ContentType.objects.get_for_model(self.instance).pk)
        setattr(form.instance, self.ct_fk_field.get_attname(),
            self.instance.pk)
        return form.save(commit=commit)



from datetime import date
import unittest

from django.test import TestCase

# In-case someone doesn't have Django REST Framework installed, guard tests.
try:
    from rest_framework import exceptions, filters
    from rest_framework.request import Request
    from rest_framework.test import APIRequestFactory

    factory = APIRequestFactory()

    from queryset_sequence.pagination import SequenceCursorPagination
except ImportError:
    factory = None
    SequenceCursorPagination = object

from queryset_sequence import QuerySetSequence

from tests.models import Author, Book, Publisher


class _TestPagination(SequenceCursorPagination):
    """A SequenceCursorPagination with a small page_size."""
    page_size = 5
    # The test models don't have a 'created' field.
    ordering = 'pages'


@unittest.skipIf(not factory, 'Must have Django REST Framework installed to run pagination tests.')
class TestSequenceCursorPagination(TestCase):
    """
    Unit tests for `queryset_sequence.pagination.SequenceCursorPagination`.

    Heavily based on `tests.test_pagination` from Django REST Framework.
    """

    PAGE_1 = [1, 2, 3, 4, 5]
    PAGE_2 = [6, 7, 8, 9, 10]
    PAGE_3 = [11, 12, 13, 14]

    def setUp(self):
        self.author = Author.objects.create(name="Jane Doe")
        self.publisher = Publisher.objects.create(name="Pablo's Publishing",
                                                  address="123 Publishing Street")

        for d in range(1, 15):
            Book.objects.create(title='Book %s' % (d % 2),
                                author=self.author,
                                publisher=self.publisher,
                                pages=d,
                                release=date(2018, 10, 5))

        self.pagination = _TestPagination()
        self.queryset = QuerySetSequence(Book.objects.filter(pages__lte=7),
                                         Book.objects.filter(pages__gt=7))

    def get_pages(self, url):
        """
        Given a URL return a tuple of:
        (previous page, current page, next page, previous url, next url)
        """
        request = Request(factory.get(url))
        queryset = self.pagination.paginate_queryset(self.queryset, request)
        current = [item.pages for item in queryset]

        next_url = self.pagination.get_next_link()
        previous_url = self.pagination.get_previous_link()

        if next_url is not None:
            request = Request(factory.get(next_url))
            queryset = self.pagination.paginate_queryset(self.queryset, request)
            next = [item.pages for item in queryset]
        else:
            next = None

        if previous_url is not None:
            request = Request(factory.get(previous_url))
            queryset = self.pagination.paginate_queryset(self.queryset, request)
            previous = [item.pages for item in queryset]
        else:
            previous = None

        return (previous, current, next, previous_url, next_url)

    def test_ordering(self):
        """Ensure that the QuerySetSequence is ordered as expected."""
        pages = [b.pages for b in self.queryset]
        self.assertEqual(pages, self.PAGE_1 + self.PAGE_2 + self.PAGE_3)

        # The first 7 are in the 0th list, the last 7 are in the 1st list.
        number = [getattr(b, '#') for b in self.queryset]
        self.assertEqual(number, [0] * 7 + [1] * 7)

    def test_invalid_cursor(self):
        request = Request(factory.get('/', {'cursor': '123'}))
        with self.assertRaises(exceptions.NotFound):
            self.pagination.paginate_queryset(self.queryset, request)

    def test_use_with_ordering_filter(self):
        class MockView:
            filter_backends = (filters.OrderingFilter,)
            ordering_fields = ['title', 'author']
            ordering = 'title'

        request = Request(factory.get('/', {'ordering': 'author'}))
        ordering = self.pagination.get_ordering(request, [], MockView())
        self.assertEqual(ordering, ('#', 'author',))

        request = Request(factory.get('/', {'ordering': '-author'}))
        ordering = self.pagination.get_ordering(request, [], MockView())
        self.assertEqual(ordering, ('#', '-author',))

        request = Request(factory.get('/', {'ordering': 'invalid'}))
        ordering = self.pagination.get_ordering(request, [], MockView())
        self.assertEqual(ordering, ('#', 'title',))

    def test_cursor_pagination(self):
        """Ensure that the cursor properly flips through pages."""
        # Check the first page.
        (previous, current, next, previous_url, next_url) = self.get_pages('/')

        self.assertIsNone(previous)
        self.assertEqual(current, self.PAGE_1)
        self.assertEqual(next, self.PAGE_2)

        # Check the second page.
        (previous, current, next, previous_url, next_url) = self.get_pages(next_url)

        self.assertEqual(previous, self.PAGE_1)
        self.assertEqual(current, self.PAGE_2)
        self.assertEqual(next, self.PAGE_3)

        # Check the third page.
        (previous, current, next, previous_url, next_url) = self.get_pages(next_url)

        self.assertEqual(previous, self.PAGE_2)
        self.assertEqual(current, self.PAGE_3)
        self.assertIsNone(next)

    def test_cursor_stableness(self):
        """
        Test what happens if we remove domains after loading a page.
        Pages should be independent, i.e. a cursor points to a particular
        domain, and shouldn't affect offsets.
        """
        (previous, current, next, previous_url, next_url) = self.get_pages('/')

        # The first page is as normal.
        self.assertIsNone(previous)
        self.assertEqual(current, self.PAGE_1)
        self.assertEqual(next, self.PAGE_2)

        # Delete Books, this shouldn't affect the next request.
        Book.objects.filter(pages__lte=3).delete()
        Book.objects.filter(pages__gte=13).delete()

        (previous, current, next, _, old_next_url) = self.get_pages(next_url)

        # Should be some missing items
        self.assertEqual(previous, [4, 5])
        self.assertEqual(current, self.PAGE_2)
        self.assertEqual(next, [11, 12])

        # Deleting some from the current page and reloading should have the page
        # offset.
        Book.objects.get(pages=7).delete()

        (previous, current, next, _, new_next_url) = self.get_pages(next_url)

        # Should be some missing items
        self.assertEqual(previous, [4, 5])
        self.assertEqual(current, [6, 8, 9, 10, 11])
        self.assertEqual(next, [12])

        # The next_url returned with the different items missing will actually
        # be different.
        self.assertNotEqual(old_next_url, new_next_url)

    def test_multiple_ordering(self):
        """Test a Pagination with multiple items in the ordering attribute."""

        # This will order by:
        #   1. Even pages
        #   2. Odd pages
        #   3. Both of the above will be done in increasing order.
        #   4. Ordering happens for 1 - 7, then 8 - 14.
        class TestPagination(_TestPagination):
            ordering = ['title', 'pages']

        self.pagination = TestPagination()

        PAGE_1 = [2, 4, 6, 1, 3]
        PAGE_2 = [5, 7, 8, 10, 12]
        PAGE_3 = [14, 9, 11, 13]

        # Check that the ordering is as expected.
        pages = [b.pages for b in self.queryset.order_by('#', *self.pagination.ordering)]
        self.assertEqual(pages, PAGE_1 + PAGE_2 + PAGE_3)

        # Now perform pretty much the same test as test_cursor_pagination, but
        # the ordering will be different.
        (previous, current, next, previous_url, next_url) = self.get_pages('/')

        self.assertIsNone(previous)
        self.assertEqual(current, PAGE_1)
        self.assertEqual(next, PAGE_2)

        (previous, current, next, previous_url, next_url) = self.get_pages(next_url)

        self.assertEqual(previous, PAGE_1)
        self.assertEqual(current, PAGE_2)
        self.assertEqual(next, PAGE_3)

        (previous, current, next, previous_url, next_url) = self.get_pages(next_url)

        self.assertEqual(previous, PAGE_2)
        self.assertEqual(current, PAGE_3)
        self.assertIsNone(next)

    def test_duplicates(self):
        """Ensure that pagination works over an 'extreme' number of duplicates."""
        PAGES = 100 # This must be unique from other fixture data.

        # Create a bunch of books that are the same.
        for i in range(15):
            Book.objects.create(title=str(i),
                                author=self.author,
                                publisher=self.publisher,
                                pages=PAGES,
                                release=date(2018, 10, 5))

        # And use only those duplicate books.
        self.queryset = QuerySetSequence(Book.objects.filter(pages=PAGES))

        titles = [item.title for item in self.queryset]

        # Look at both the pages (which should all be 1) and the IDs.
        next_url = '/'
        for i in range(3):
            request = Request(factory.get(next_url))
            queryset = self.pagination.paginate_queryset(self.queryset, request)
            titles, pages = zip(*[(item.title, item.pages) for item in queryset])

            self.assertEqual(titles, tuple(map(lambda d: str(d + (i * 5)), [0, 1, 2, 3, 4])))
            self.assertEqual(pages, (PAGES, ) * 5)

            next_url = self.pagination.get_next_link()

        self.assertIsNone(next_url)



from __future__ import absolute_import

import collections
import functools
import json
import logging
import re
import time
import unittest

from django.conf import settings
from django.contrib.admin.sites import site as admin_site
from django.utils import six

from selenium.webdriver.common.action_chains import ActionChains

from django_admin_testutils import AdminSeleniumTestCase


logger = logging.getLogger(__name__)


if not hasattr(__builtins__, 'cmp'):
    def cmp(a, b):
        return (a > b) - (a < b)


def is_sequence(o):
    return isinstance(o, collections.Sequence)


def is_integer(o):
    return isinstance(o, six.integer_types)


def is_str(o):
    return isinstance(o, six.string_types)


get_model_name = lambda m: "-".join([m._meta.app_label, m._meta.model_name])


def xpath_cls(classname):
    return 'contains(concat(" ", @class, " "), " %s ")' % classname


def xpath_item(model_name=None):
    xpath_item_predicate = 'not(contains(@class, "-drag")) and not(contains(@class, "thead"))'
    expr = "%s and %s" % (xpath_cls('djn-item'), xpath_item_predicate)
    if model_name:
        expr += ' and contains(@class, "-%s")' % model_name
    return expr


class BaseNestedAdminTestCase(AdminSeleniumTestCase):

    root_urlconf = "nested_admin.tests.urls"

    root_model = None
    nested_models = None

    @classmethod
    def setUpClass(cls):
        super(BaseNestedAdminTestCase, cls).setUpClass()

        root_admin = admin_site._registry[cls.root_model]

        def descend_admin_inlines(admin):
            data = [admin.model, []]
            for inline in (getattr(admin, 'inlines', None) or []):
                data[1].append(descend_admin_inlines(inline))
            return data

        cls.models = descend_admin_inlines(root_admin)

        def recursive_map_model_names(data):
            if isinstance(data, list):
                return [m for m in map(recursive_map_model_names, data)]
            else:
                return get_model_name(data)

        cls.model_names = recursive_map_model_names(cls.models)

    def load_admin(self, obj=None):
        if obj is None:
            obj = self.root_model
        super(BaseNestedAdminTestCase, self).load_admin(obj)

    def save_form(self):
        browser_errors = [e for e in self.selenium.get_log('browser')
                          if 'favicon' not in e['message']]
        if len(browser_errors) > 0:
            logger.error("Found browser errors: %s" % json.dumps(browser_errors, indent=4))
        super(BaseNestedAdminTestCase, self).save_form()

    def _normalize_indexes(self, indexes, is_group=False, named_models=True):
        """
        To allow for the case where there are multiple inlines defined on a
        given ModelAdmin or InlineModelAdmin, we need indexes to be a list of
        lists (or at any rate an iterable of iterables, but lets call the
        outermost iterable a list, and the elements themselves tuples).
        The elements of the top-level list should be seen as directions for
        how to descend the hierarchy of the inlines (an empty list thus
        referring to the root model, which is not an inline). Descending down
        the hierarchy, each element is a 2-tuple (a, b) where ``a``
        is the index on the ModelAdmin.inlines attribute, and b is a 0-indexed
        number corresponding to the actual item in that inline currently on
        the page.

        Some operations, such as ``add_inline`` and ``get_num_inlines`` (along
        with the helper method ``get_group``) refer not to a specific item
        within an inline, but to the group of inlines as a whole (at least
        at the last step of descending the hierarchy: for instance when
        clicking add on a nested inline; the add button is at the group-level,
        but the parents of that group are all specific inline items). In the
        case of such operations that operate at the group level, the last item
        is a 1-tuple to indicate which group of the available inlines at that
        level is the target.

        In the trivial case, there is only one inline at every level. Most of
        the unit tests are trivial in this sense. It would be very burdensome,
        not to mention difficult to parse, if the operation::

            self.add_inline(indexes=[1, 3, 2])

        had to be written out as::

            self.add_inline(indexes=[(0, 1), (0, 3), (0, 2), (0, ))])

        The first example above is hard enough to follow without adding a
        number of zeros to each element (and at the end!) to accommodate the
        possibility of there being multiple inline classes when we already now
        that there is only one.

        To ease the burden of writing test cases, it is possible to use an
        integer at a given level of the indexes if there is only one possible
        inline at that level. It is similarly alright to omit the terminal
        1-tuple of (0, ) to ``add_inline`` and ``get_num_inlines`` if there
        is only one possible inline group that can be added to.

        The purposes of this function is to normalize the indexes parameter
        so that it can be converted from the shorter notation to a fully
        qualified one of 2-tuples (and the occasional terminal 1-tuple).
        """
        norm_indexes = []
        root_model_name, inline_model_names = list(self.model_names)

        indexes = list(indexes or [])

        # Are we already normalized? If so, just return
        inline_type_check = is_str if named_models else is_integer
        if is_group:
            if len(indexes) and inline_type_check(indexes[-1]):
                try:
                    if all([inline_type_check(a) and is_integer(b) for a, b in indexes[:-1]]):
                        return indexes
                except:
                    pass
        else:
            try:
                if all([inline_type_check(a) and is_integer(b) for a, b in indexes]):
                    return indexes
            except:
                pass

        group_index = None
        if is_group:
            if len(indexes) and is_sequence(indexes[-1]) and len(indexes[-1]) == 1:
                group_index = indexes.pop()[0]
            else:
                indexes.append(None)
        elif not indexes:
            return indexes

        for level, level_indexes in enumerate(indexes):
            if len(inline_model_names) == 0:
                raise ValueError("Indexes depth greater than inline depth")
            if level_indexes is None:
                if not is_group:
                    raise ValueError("Unexpected None found in indexes")
                if len(inline_model_names) > 1:
                    raise ValueError(
                        "Terminal index to inline class omitted in group-level "
                        "operation, but parent has more than one inline")
                if named_models:
                    norm_indexes.append(inline_model_names[0][0])
                else:
                    norm_indexes.append(0)
                break
            if not is_sequence(level_indexes) and not is_integer(level_indexes):
                raise ValueError("Unexpected type %s in list of indexes" % (
                    type(level_indexes).__name__))
            if is_integer(level_indexes):
                if len(inline_model_names) > 1:
                    raise ValueError((
                        "indexes[%d] using shorthand integer value, but more "
                        "than one inline to choose from") % (level))
                level_indexes = [0, level_indexes]
            if is_sequence(level_indexes):
                if len(level_indexes) != 2:
                    raise ValueError("Expected indexes[%d] to have len 2, got %d" % (
                        level, len(level_indexes)))
                if not all([is_integer(i) for i in level_indexes]):
                    raise ValueError("indexes[%d] must have only integers" % level)
                inline_index, inline_item = level_indexes
                inline_model_name, inline_model_names = inline_model_names[inline_index]
                if named_models:
                    norm_indexes.append([inline_model_name, inline_item])
                else:
                    norm_indexes.append([inline_index, inline_item])

        if group_index is not None:
            if named_models:
                norm_indexes.append(inline_model_names[group_index][0])
            else:
                norm_indexes.append(group_index)

        return norm_indexes

    def drag_and_drop_item(self, from_indexes, to_indexes, screenshot_hack=False):
        action = DragAndDropAction(self, from_indexes=from_indexes, to_indexes=to_indexes)
        action.move_to_target(screenshot_hack=screenshot_hack)

    def get_num_inlines(self, indexes=None):
        indexes = self._normalize_indexes(indexes, is_group=True)
        model_name = indexes[-1]
        group = self.get_group(indexes=indexes)
        group_id = group.get_attribute('id')
        selector = "#%s .djn-dynamic-form-%s" % (group_id, model_name)
        return self.selenium.execute_script("return $('%s').length" % selector)

    def get_group(self, indexes=None):
        indexes = self._normalize_indexes(indexes, is_group=True)
        model_name = indexes.pop()
        expr_parts = []
        for parent_model, parent_item_index in indexes:
            expr_parts += ["/*[%s][%d]" % (xpath_item(parent_model), parent_item_index + 1)]
        expr_parts += ["/*[@data-inline-model='%s']" % model_name]
        expr = "/%s" % ("/".join(expr_parts))
        return self.selenium.find_element_by_xpath(expr)

    def get_item(self, indexes):
        indexes = self._normalize_indexes(indexes)
        model_name, item_index = indexes.pop()
        indexes.append(model_name)
        group = self.get_group(indexes=indexes)
        return group.find_element_by_xpath(".//*[%s][%d]" % (xpath_item(model_name), item_index + 1))

    def add_inline(self, indexes=None, name=None, slug=None):
        indexes = self._normalize_indexes(indexes, is_group=True)
        new_index = self.get_num_inlines(indexes)
        model_name = indexes[-1]
        add_selector = "#%s .djn-add-item a.djn-add-handler.djn-model-%s" % (
            self.get_group(indexes).get_attribute('id'), model_name)
        with self.clickable_selector(add_selector) as el:
            el.click()
        indexes[-1] = [model_name, new_index]
        if name is not None:
            self.set_field("name", name, indexes=indexes)
        if slug is not None:
            self.set_field("slug", slug, indexes=indexes)

    def remove_inline(self, indexes):
        indexes = self._normalize_indexes(indexes)
        model_name = indexes[-1][0]
        remove_selector = "#%s .djn-remove-handler.djn-model-%s" % (
            self.get_item(indexes).get_attribute('id'), model_name)
        with self.clickable_selector(remove_selector) as el:
            el.click()

    def delete_inline(self, indexes):
        indexes = self._normalize_indexes(indexes)
        model_name = indexes[-1][0]
        item_id = self.get_item(indexes).get_attribute('id')
        delete_selector = "#%s .djn-delete-handler.djn-model-%s" % (
            item_id, model_name)
        with self.clickable_selector(delete_selector) as el:
            el.click()
        if self.has_grappelli:
            undelete_selector = "#%s.grp-predelete .grp-delete-handler.djn-model-%s" % (
                item_id, model_name)
            self.wait_until_clickable_selector(undelete_selector)

    def undelete_inline(self, indexes):
        indexes = self._normalize_indexes(indexes)
        model_name = indexes[-1][0]
        item_id = self.get_item(indexes).get_attribute('id')
        undelete_selector = "#%s .djn-delete-handler.djn-model-%s" % (item_id, model_name)
        with self.clickable_selector(undelete_selector) as el:
            el.click()
        if self.has_grappelli:
            delete_selector = "#%s:not(.grp-predelete) .grp-delete-handler.djn-model-%s" % (
                item_id, model_name)
            self.wait_until_clickable_selector(delete_selector)

    def get_form_field_selector(self, attname, indexes=None):
        indexes = self._normalize_indexes(indexes)
        if not indexes:
            return "#id_%s" % attname
        field_prefix = self.get_item(indexes=indexes).get_attribute('id')
        return "#id_%s-%s" % (field_prefix, attname)

    def get_field(self, attname, indexes=None):
        indexes = self._normalize_indexes(indexes)
        field_selector = self.get_form_field_selector(attname, indexes=indexes)
        return self.selenium.find_element_by_css_selector(field_selector)

    def set_field(self, attname, value, indexes=None):
        indexes = self._normalize_indexes(indexes)
        field_selector = self.get_form_field_selector(attname, indexes=indexes)
        with self.clickable_selector(field_selector) as el:
            el.clear()
            el.send_keys(value)


class DragAndDropAction(object):

    def __init__(self, test_case, from_indexes, to_indexes):
        self.test_case = test_case
        self.selenium = test_case.selenium

        if len(from_indexes) > len(to_indexes):
            self.target_is_empty = True
        else:
            self.target_is_empty = False

        self.from_indexes = self.test_case._normalize_indexes(from_indexes, named_models=False)
        self.to_indexes = self.test_case._normalize_indexes(
            to_indexes, is_group=self.target_is_empty, named_models=False)

        num_inlines_indexes = self.test_case._normalize_indexes(
            to_indexes, is_group=self.target_is_empty, named_models=True)
        if not is_integer(num_inlines_indexes[-1]):
            num_inlines_indexes[-1] = num_inlines_indexes[-1][0]
        self.target_num_items = self.test_case.get_num_inlines(num_inlines_indexes)

        if is_integer(self.to_indexes[-1]):
            self.to_indexes[-1] = [self.to_indexes[-1], 0]
        self.to_indexes = [tuple(i) for i in self.to_indexes]

        inline_models = self.test_case.models[1]
        for inline_index, item_index in self.to_indexes[:-1]:
            inline_models = inline_models[inline_index][1]
        self.target_num_inlines = len(inline_models)

        if self.from_indexes[:-1] == self.to_indexes[:-1]:
            if self.from_indexes[-1][1] < self.to_indexes[-1][1]:
                self.to_indexes[-1][1] += 1

        self.test_case.assertEqual(len(to_indexes), len(from_indexes),
            "Depth of source and target must be the same")

    @property
    def source(self):
        if not hasattr(self, '_source'):
            source_item = self.test_case.get_item(indexes=self.from_indexes)
            if source_item.tag_name == 'div':
                drag_handler_xpath = "h3"
            elif self.test_case.has_grappelli:
                drag_handler_xpath = "/".join([
                    "*[%s]" % xpath_cls("djn-tr"),
                    "*[%s]" % xpath_cls("djn-td"),
                    "*[%s]" % xpath_cls("tools"),
                    "/*[%s]" % xpath_cls("djn-drag-handler"),
                ])
            else:
                drag_handler_xpath = "/".join([
                    "*[%s]" % xpath_cls("djn-tr"),
                    "*[%s]" % xpath_cls("is-sortable"),
                    "*[%s]" % xpath_cls("djn-drag-handler"),
                ])
            self._source = source_item.find_element_by_xpath(drag_handler_xpath)
        return self._source

    @property
    def target(self):
        if not hasattr(self, '_target'):
            if len(self.to_indexes) > 1:
                target_inline_parent = self.test_case.get_item(self.to_indexes[:-1])
            else:
                target_inline_parent = self.selenium
            target_xpath = ".//*[%s][%d]//*[%s]" % (
                xpath_cls('djn-group'), self.to_indexes[-1][0] + 1, xpath_cls("djn-items"))
            if self.target_num_items != self.to_indexes[-1][1]:
                target_xpath += "/*[%(item_pred)s][%(item_pos)d]" % {
                    'item_pred': xpath_item(),
                    'item_pos': self.to_indexes[-1][1] + 1,
                }
            self._target = target_inline_parent.find_element_by_xpath(target_xpath)
        return self._target

    def initialize_drag(self):
        source = self.source
        if self.test_case.has_suit and self.test_case.browser == 'chrome' and source.tag_name == 'h3':
            source = source.find_element_by_css_selector('b')
        (ActionChains(self.selenium)
            .move_to_element_with_offset(source, 0, 0)
            .click_and_hold()
            .perform())
        time.sleep(0.05)
        ActionChains(self.selenium).move_by_offset(0, -15).perform()
        time.sleep(0.05)
        ActionChains(self.selenium).move_by_offset(0, 15).perform()
        with self.test_case.visible_selector('.ui-sortable-helper') as el:
            return el

    def move_by_offset(self, x_offset, y_offset):
        increment = -15 if y_offset < 0 else 15
        for offset in range(0, y_offset, increment):
            ActionChains(self.selenium).move_by_offset(0, increment).perform()
        if offset != y_offset:
            ActionChains(self.selenium).move_by_offset(0, y_offset - offset).perform()

    def release(self):
        ActionChains(self.selenium).release().perform()

    def _match_helper_with_target(self, helper, target):
        limit = 8
        count = 0
        # True if aiming for the bottom of the target
        helper_height = helper.size['height']
        self.move_by_offset(0, -1)
        desired_pos = tuple(self.to_indexes)
        target_bottom = bool(0 < self.to_indexes[-1][1] == (self.target_num_items - 1)
            and self.to_indexes[-1][0] == (self.target_num_inlines - 1)
            and cmp(desired_pos[:-1], self.current_position[:-1]) > -1)

        while True:
            helper_y = helper.location['y']
            y_offset = target.location['y'] - helper_y
            if target_bottom:
                if y_offset > 0:
                    y_offset += target.size['height']
                else:
                    y_offset += target.size['height'] - helper_height
            if abs(y_offset) < 1:
                break
            if count >= limit:
                break
            scaled_offset = int(round(y_offset * 0.9))
            self.move_by_offset(0, scaled_offset)
            if count == 0 and helper_y == helper.location['y']:
                # phantomjs: the drag didn't have any effect.
                # refresh the action chain
                self.release()
                time.sleep(0.1)
                helper = self.initialize_drag()
                time.sleep(0.1)
                self.move_by_offset(0, scaled_offset)
            curr_pos = self.current_position
            pos_diff = cmp(desired_pos, curr_pos)
            if pos_diff == 0:
                break
            count += 1
        return helper

    def _num_preceding_siblings(self, ctx, condition):
        """
        For an unknown reason, evaluating XPath expressions of the form

            preceding-sibling::*[not(contains(@attr, 'value'))]

        Where 'value' is contained in at least one of the preceding siblings,
        is extraordinarily slow. So we just grab all siblings and iterate
        through the elements in python.
        """
        siblings = ctx.find_element_by_xpath('parent::*').find_elements_by_xpath('*')
        count = 0
        for el in siblings:
            if el.id == ctx.id:
                break
            if condition(el):
                count += 1
        return count

    def _num_preceding_djn_items(self, ctx):
        def is_djn_item(el):
            classes = set(re.split(r'\s+', el.get_attribute('class')))
            return (classes & {'djn-item'} and
                not(classes & {'djn-no-drag', 'djn-thead', 'djn-item-dragging'}))

        return self._num_preceding_siblings(ctx, condition=is_djn_item)

    def _num_preceding_djn_groups(self, ctx):
        def is_djn_group(el):
            return "djn-group" in re.split(r'\s+', el.get_attribute('class'))

        return self._num_preceding_siblings(ctx, condition=is_djn_group)

    @property
    def current_position(self):
        placeholder = self.selenium.find_element_by_css_selector(
            '.ui-sortable-placeholder')
        pos = []
        ctx = None
        ancestor_xpath = 'ancestor::*[%s][1]' % xpath_cls("djn-item")
        for i in range(0, len(self.to_indexes)):
            if ctx is None:
                ctx = placeholder
            else:
                ctx = ctx.find_element_by_xpath(ancestor_xpath)
            item_index = self._num_preceding_djn_items(ctx)
            ctx = ctx.find_element_by_xpath('ancestor::*[%s][1]' % xpath_cls("djn-group"))
            inline_index = self._num_preceding_djn_groups(ctx)
            pos.insert(0, (inline_index, item_index))
        return tuple(pos)

    def _finesse_position(self, helper, target):
        limit = 200
        count = 0
        helper_height = helper.size['height']
        if self.target_num_items == 0:
            dy = max(1, int(round(helper_height / 38.0)))
        else:
            dy = int(round(helper_height / 10.0))
        desired_pos = tuple(self.to_indexes)
        while True:
            if count >= limit:
                break
            curr_pos = self.current_position
            pos_diff = cmp(desired_pos, curr_pos)
            if pos_diff == 0:
                break
            self.move_by_offset(0, pos_diff * dy)
            count += 1

    def move_to_target(self, screenshot_hack=False):
        target = self.target
        helper = self.initialize_drag()
        if screenshot_hack and 'phantomjs' in self.test_case.browser:
            # I don't know why, but saving a screenshot fixes a weird bug
            # in phantomjs
            self.selenium.save_screenshot('/dev/null')
        helper = self._match_helper_with_target(helper, target)
        self._finesse_position(helper, target)
        self.release()


def expected_failure_if_grappelli(func):
    if 'grappelli' in settings.INSTALLED_APPS:
        return unittest.expectedFailure(func)
    return func


def expected_failure_if_suit(func):
    if 'suit' in settings.INSTALLED_APPS:
        return unittest.expectedFailure(func)
    return func


def skip_if_not_grappelli(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if 'grappelli' not in settings.INSTALLED_APPS:
            raise unittest.SkipTest("Skipping (grappelli required)")
        return func(*args, **kwargs)
    return wrapper



from django.conf.urls.defaults import *


urlpatterns = patterns('example.site_app.views',
    url(r'^photo/$', 'create_photo', name='create_photo'),
    url(r'^post/$', 'create_post', name='create_post'),
)



from django.contrib import admin
from nested_admin import NestedGenericStackedInline, NestedModelAdmin
from .models import GFKRoot, GFKA, GFKB


class GFKBInline(NestedGenericStackedInline):
    model = GFKB
    extra = 0
    sortable_field_name = "position"
    inline_classes = ("collapse", "open", "grp-collapse", "grp-open", )


class GFKAInline(NestedGenericStackedInline):
    model = GFKA
    extra = 0
    sortable_field_name = "position"
    inlines = [GFKBInline]
    inline_classes = ("collapse", "open", "grp-collapse", "grp-open", )


@admin.register(GFKRoot)
class GFKRootAdmin(NestedModelAdmin):
    inlines = [GFKAInline]



from __future__ import unicode_literals

from django.db import models
from django.db.models import ForeignKey, CASCADE
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class RootAbstract(models.Model):

    slug = models.CharField(max_length=128)

    class Meta:
        abstract = True

    def __str__(self):
        return self.slug


@python_2_unicode_compatible
class ChildAbstract(models.Model):

    slug = models.CharField(max_length=128, help_text="Halp")
    position = models.PositiveIntegerField()
    boolean = models.BooleanField(default=False)
    readonly = models.CharField(max_length=255)
    text = models.TextField(default='')

    class Meta:
        abstract = True

    def __str__(self):
        parts = ["%s[%d]" % (self.slug, self.position)]
        if self.root:
            parts.insert(0, "%s" % self.root)
        return "/".join(parts)


class PlainStackedRoot(RootAbstract):

    class Meta:
        verbose_name = "Stacked Root"
        verbose_name_plural = "Stacked Roots"


class PlainStackedChild(ChildAbstract):
    root = ForeignKey(PlainStackedRoot, related_name='children', on_delete=CASCADE)

    class Meta:
        ordering = ('root', 'position')
        verbose_name = "Stacked Child"
        verbose_name_plural = "Stacked Children"

class PlainTabularRoot(RootAbstract):

    class Meta:
        verbose_name = "Tabular Root"
        verbose_name_plural = "Tabular Roots"



class PlainTabularChild(ChildAbstract):
    root = ForeignKey(PlainTabularRoot, related_name='children', on_delete=CASCADE)

    class Meta:
        ordering = ('root', 'position')
        verbose_name = "Tabular Child"
        verbose_name_plural = "Tabular Children"


class NestedStackedRoot(RootAbstract):

    class Meta:
        verbose_name = "Stacked Root"
        verbose_name_plural = "Stacked Roots"


class NestedStackedChild(ChildAbstract):
    root = ForeignKey(NestedStackedRoot, related_name='children', on_delete=CASCADE)

    class Meta:
        ordering = ('root', 'position')
        verbose_name = "Stacked Child"
        verbose_name_plural = "Stacked Children"


class NestedTabularRoot(RootAbstract):

    class Meta:
        verbose_name = "Tabular Root"
        verbose_name_plural = "Tabular Roots"


class NestedTabularChild(ChildAbstract):
    root = ForeignKey(NestedTabularRoot, related_name='children', on_delete=CASCADE)

    class Meta:
        ordering = ('root', 'position')
        verbose_name = "Tabular Child"
        verbose_name_plural = "Tabular Children"
